@@ -77,7 +77,7 @@ static int new_x, new_y;
 int
 mousegetch ()
 {
-	int stat, x, y;
+	int stat;
 
 	showmouse ();
 	mousestat ();
