@@ -6,11 +6,16 @@
    * character cells. */
 
 extern int width_data_version;
+#define cjk_width_data 64
+
 extern int nonbmp_width_data;
+
 #define plane_2_double_width	((nonbmp_width_data & 0x4) != 0)
 #define plane_1_combining	((nonbmp_width_data & 0x2) == 0)
 #define plane_14_combining	((nonbmp_width_data & 0x1) == 0)
 
+extern enum {False, True} utf8_screen;
+
 struct interval {
   unsigned int first;
   unsigned int last;
@@ -18,8 +23,8 @@ struct interval {
 
 static struct interval
 ambiguous [] = {
-    /* version since xterm 167; there was a different version in 
-       earlier xterm releases, but unused then */
+    /* version since xterm 167 (Unicode 3.2); there was a different 
+       version in earlier xterm releases, but unused then */
     { 0x00A1, 0x00A1 }, { 0x00A4, 0x00A4 }, { 0x00A7, 0x00A8 },
     { 0x00AA, 0x00AA }, { 0x00AD, 0x00AE }, { 0x00B0, 0x00B4 },
     { 0x00B6, 0x00BA }, { 0x00BC, 0x00BF }, { 0x00C6, 0x00C6 },
@@ -74,7 +79,7 @@ ambiguous [] = {
 };
 static struct interval
 ambiguous180 [] = {
-    /* version since xterm 180 */
+    /* version since xterm 180 (Unicode 4.0) */
     { 0x00A1, 0x00A1 }, { 0x00A4, 0x00A4 }, { 0x00A7, 0x00A8 },
     { 0x00AA, 0x00AA }, { 0x00AE, 0x00AE }, { 0x00B0, 0x00B4 },
     { 0x00B6, 0x00BA }, { 0x00BC, 0x00BF }, { 0x00C6, 0x00C6 },
@@ -159,7 +164,7 @@ iswide (ucs)
   unsigned long ucs;
 {
   if (ucs & 0x80000000) {
-    /* special encoding of 2 Unicode chars, mapped from 1 JIS character */
+    /* special encoding of 2 Unicode chars, mapped from 1 CJK character */
     if ((ucs & 0xFFF3) == 0x02E1) {
       /* 0x02E9 0x02E5 or 0x02E5 0x02E9 */
       return 1;
@@ -169,18 +174,20 @@ iswide (ucs)
     }
   }
 
-  if (width_data_version == 4 &&
-      bisearch ((unsigned int) ucs, 
-                ambiguous180, 
-                sizeof (ambiguous180) / sizeof (struct interval) - 1))
+  if (width_data_version == (cjk_width_data | 2)
+      && (utf8_screen || ucs >= 0x100)
+      && bisearch ((unsigned int) ucs, 
+                   ambiguous180, 
+                   sizeof (ambiguous180) / sizeof (struct interval) - 1))
   {
     return 1;
   }
 
-  if (width_data_version == 3 &&
-      bisearch ((unsigned int) ucs, 
-                ambiguous, 
-                sizeof (ambiguous) / sizeof (struct interval) - 1))
+  if (width_data_version == (cjk_width_data | 1)
+      && (utf8_screen || ucs >= 0x100)
+      && bisearch ((unsigned int) ucs, 
+                   ambiguous, 
+                   sizeof (ambiguous) / sizeof (struct interval) - 1))
   {
     return 1;
   }
@@ -195,7 +202,7 @@ iswide (ucs)
 
   if (width_data_version == 1) {
    return
-    /* version since xterm 157 until 166 */
+    /* Unicode 3.2, xterm 157 until 166, wcwidth 2001/1/12 */
     (ucs >= 0x1100 && ucs <= 0x115f) || /* Hangul Jamo */
     (ucs >= 0x2e80 && ucs <= 0xa4cf && (ucs & (unsigned long) ~0x0011) != 0x300a &&
      ucs != 0x303f) ||                  /* CJK ... Yi */
@@ -208,7 +215,7 @@ iswide (ucs)
     ;
   } else {
    return
-    /* version since xterm 167 */
+    /* since xterm 167, wcwidth 2002-05-18 */
     /* 0x3xxxx range included since xterm 180 */
     (ucs >= 0x1100 &&
      (ucs <= 0x115f ||                    /* Hangul Jamo init. consonants */
