@@ -17,11 +17,12 @@
 
 
 voidfunc keyproc = I;	/* function addressed by entered function key */
+unsigned char keyshift = 0;	/* shift state of entered function key */
+static unsigned char fkeyshift;
 #define max_ansi_params 10
 int ansi_params;
 int ansi_param [max_ansi_params];
 char ansi_fini;
-char keyshift = '0';	/* shift state of entered function key */
 
 static
 int keymap_delay = 0;		/* wait to detect mapped key sequence */
@@ -36,38 +37,60 @@ int default_escape_delay = 450;	/* overridden by $ESCDELAY */
 static
 int dont_suppress_keymap = 1;
 
+static
+int report_winchg = 0;
+
 /* The flag queue_mode_mapped indicates when the byte input queue 
    has been stuffed with keyboard mapped characters, always being 
    encoded in UTF-8, while otherwise the queue contents are in 
-   terminal native encoding (which may also be UTF-8, or CJK, or Latin-1)
+   terminal native encoding (which may also be UTF-8, or CJK, or Latin-1).
  */
 static
 FLAG queue_mode_mapped = False;	/* queue contains mapped character(s) */
 
+/* In hp_shift_mode key shift indications like ESC 2p or ESC 5h 
+   are considered to determine the keyshift state.
+ */
+static
+FLAG hp_shift_mode = False;
+
+/* In sco_shift_mode key shift indications like ESC [2A or ESC [2M 
+   are considered to determine the keyshift state.
+ */
+static
+FLAG sco_shift_mode = False;
+
+
+/*======================================================================*\
+|*		Local function declaration				*|
+\*======================================================================*/
+
+static unsigned long _readchar _((void));
+
 
 /*======================================================================*\
 |*		Handling function key and keyboard mapping tables	*|
 \*======================================================================*/
 
-#ifndef msdos
-
 #define in_status_line	(current_cursor_y == YMAX)
 
+#ifndef msdos
+
 static
 void
 dummyfunc ()
 {
 }
 
-extern struct fkeyentry {
-	char * fk;
-	voidfunc fp;
-} fkeymap [];
-
+extern struct fkeyentry fkeymap [];
 extern struct fkeyentry fkeymap_xterm [];
+extern struct fkeyentry fkeymap_rxvt [];
 extern struct fkeyentry fkeymap_vt100 [];
+extern struct fkeyentry fkeymap_vt52 [];
 extern struct fkeyentry fkeymap_hp [];
 extern struct fkeyentry fkeymap_siemens [];
+extern struct fkeyentry fkeymap_scoansi [];
+extern struct fkeyentry fkeymap_interix [];
 
 struct fkeyentry * fkeymap2 = fkeymap_xterm;
 
@@ -79,12 +102,24 @@ set_fkeymap (term)
 	return;
   } else if (* term == 'x') {
 	fkeymap2 = fkeymap_xterm;
+  } else if (* term == 'r') {
+	fkeymap2 = fkeymap_rxvt;
   } else if (* term == 'h') {
 	fkeymap2 = fkeymap_hp;
+	hp_shift_mode = True;
   } else if (* term == 'v') {
 	fkeymap2 = fkeymap_vt100;
+  } else if (* term == '5') {
+	fkeymap2 = fkeymap_vt52;
+  } else if (* term == 'i') {
+	fkeymap2 = fkeymap_interix;
   } else if (* term == 's') {
 	fkeymap2 = fkeymap_siemens;
+	use_mouse = False;
+  } else if (* term == 'o') {
+	fkeymap2 = fkeymap_scoansi;
+	sco_shift_mode = True;
+	use_mouse = False;
   }
 }
 
@@ -106,6 +141,34 @@ static char * next_keyboard_mapping = "";
 static keymaptabletype keyboard_map = (keymaptabletype) NIL_PTR;
 static keymaptabletype last_keyboard_map = (keymaptabletype) NIL_PTR;
 
+/**
+   Notify user of changed input method
+ */
+static
+void
+notify_input_method ()
+{
+  menuitemtype * km = lookup_Keymap_menuitem (keyboard_mapping);
+  char * source = NIL_PTR;
+
+  switch (km->extratag ? km->extratag [0] : ' ') {
+	case 'U':	source = "UnicodeData"; break;
+	case 'H':	source = "Unihan"; break;
+	case 'C':	source = "cxterm"; break;
+	case 'M':	source = "m17n"; break;
+	case 'Y':	source = "yudit"; break;
+	case 'V':	source = "vim"; break;
+	case 'X':	source = "X"; break;
+  }
+  if (source) {
+	build_string (text_buffer, "Input method: %s (source: %s)", 
+			km->itemname, source);
+  } else {
+	build_string (text_buffer, "Input method: %s", km->itemname);
+  }
+  status_msg (text_buffer);
+}
+
 /**
    Set keymap.
  */
@@ -121,6 +184,10 @@ set_keymap (new_keymap, current, next)
   keyboard_map = new_keymap;
   keyboard_mapping = current;
   next_keyboard_mapping = next;
+
+  if (! loading && ! in_status_line) {
+	notify_input_method ();
+  }
 }
 
 /**
@@ -136,7 +203,7 @@ toggleKEYMAP ()
 	} else {
 		set_keymap (last_keyboard_map, last_keyboard_mapping, last_keyboard_mapping);
 	}
-	displayflags ();
+	flags_changed = True;
   } else {
 	error ("Keyboard mapping not active");
   }
@@ -154,7 +221,7 @@ selectKEYMAP (script)
 
   if (script == NIL_PTR || * script == '\0') {
 	set_keymap ((keymaptabletype) NIL_PTR, "--", keymappingtable [0].shortcut);
-	displayflags ();
+	flags_changed = True;
 	return True;
   } else {
 	/* map alternative shortcuts */
@@ -170,7 +237,7 @@ selectKEYMAP (script)
 			    keymappingtable [i].shortcut,
 			    i + 1 == keymappingtable_len ?
 			      "" : keymappingtable [i + 1].shortcut);
-		displayflags ();
+		flags_changed = True;
 		return True;
 	    }
 	}
@@ -200,6 +267,7 @@ setKEYMAP (script)
   }
 }
 
+#ifdef not_used
 /**
    Cycle through keyboard mappings.
  */
@@ -208,6 +276,7 @@ cycleKEYMAP ()
 {
   selectKEYMAP (next_keyboard_mapping);
 }
+#endif
 
 /**
    Enable keyboard mapping interactively.
@@ -269,7 +338,7 @@ get_digits (result)
   register int index;
   register int count;
 
-  index = _readchar_nokeymap ();
+  index = read1byte ();
   if (index == quit_char) {
 	quit = True;
   }
@@ -280,7 +349,7 @@ get_digits (result)
 	count *= 10;
 	count += index - '0';
 	* result = count;
-	index = _readchar_nokeymap ();
+	index = read1byte ();
 	if (index == quit_char) {
 		quit = True;
 	}
@@ -355,11 +424,10 @@ timestamp_readchar ()
  * an improvement.
  */
 static
-int
+unsigned long
 readchar ()
 {
-  register character c;
-  int i;
+  unsigned long c;
 
 #ifdef msdos
 #define winchhere
@@ -379,7 +447,7 @@ readchar ()
   waitingforinput = True;
 
   c = _readchar ();
-  while (c == (character) FUNcmd && (* keyproc) == MOUSEescape 
+  while (command (c) == MOUSEescape
 	&& mouse_button == releasebutton && report_release == False)
   {
 	c = _readchar ();
@@ -404,14 +472,6 @@ readchar ()
 
   timestamp_readchar ();
 
-  if (pc_term && utf8_text && utf8_input == False && c >= 0x80) {
-	for (i = 0; i < 128; i ++) {
-		if (c == (character) pc_charmap [i]) {
-			return (character) (i + 128);
-		}
-	}
-  }
-
   return c;
 }
 
@@ -420,38 +480,51 @@ readchar ()
    but doesn't apply international keyboard mapping
  */
 static
-int
+unsigned long
 readchar_nokeymap ()
 {
-  int res;
+  unsigned long c;
   int prev_dont_suppress_keymap = dont_suppress_keymap;
 
   dont_suppress_keymap = 0;
-  res = readchar ();
+  c = readchar ();
   dont_suppress_keymap = prev_dont_suppress_keymap;
 
-  return res;
+  return c;
 }
 
 /*
    readcharacter_allbuttons reads like readcharacter
    but always reports mouse button release
+   and also reports window size change as function RDwin
  */
 unsigned long
 readcharacter_allbuttons ()
 {
+  unsigned long c;
+  int prev_report_winchg = report_winchg;
+
+  report_winchg = 1;
+
   report_release = True;
-  return readcharacter ();
+  c = readcharacter ();
+
+  report_winchg = prev_report_winchg;
+
+  return c;
 }
 
+#define dont_debug_mapped_keyboard
+
 /*
-   readcharacter_mapping () reads one Unicode character
+   readcharacter_mapping () reads one character
    either with or without keyboard mapping
  */
 static
 unsigned long
-readcharacter_mapping (map_keyboard)
+readcharacter_mapping (map_keyboard, return_unicode)
   FLAG map_keyboard;
+  FLAG return_unicode;
 {
   int utfcount = 1;
   unsigned long unichar;
@@ -461,14 +534,17 @@ readcharacter_mapping (map_keyboard)
 	unichar = readchar ();
   } else {
 	unichar = readchar_nokeymap ();
-	if (unichar == (character) FUNcmd &&
-		(     (* keyproc) == SNL
-		/* || (* keyproc) == Popmark */
-		)) {
+	/* double check needed here because command ('\n') == SNL */
+	if (unichar == FUNcmd && command (unichar) == SNL) {
 		return '\r';
 	}
   }
 
+  if (unichar == FUNcmd) {
+	/* pass back WINCH exception */
+	return FUNcmd;
+  }
+
   if (utf8_input || queue_mode_mapped) {
 	if ((unichar & 0x80) == 0x00) {
 		utfcount = 1;
@@ -494,57 +570,115 @@ readcharacter_mapping (map_keyboard)
 		unichar = (unichar << 6) | (readchar_nokeymap () & 0x3F);
 		utfcount --;
 	}
-	if (cjk_text || mapped_text) {
-		return cjk (unichar);
+	if (! return_unicode && (cjk_text || mapped_text)) {
+		return encodedchar (unichar);
 	} else {
 		return unichar;
 	}
   } else if (cjk_term && multichar (unichar)) {
-	if (cjk_encoding == 'C' && unichar == 0x8E) {
-		return unichar << 24
+	if (text_encoding_tag == 'C' && unichar == 0x8E) {
+		unichar = (unichar << 24)
 			| (readchar_nokeymap () << 16)
 			| (readchar_nokeymap () << 8)
 			| readchar_nokeymap ();
-	} else if (cjk_encoding == 'J' && unichar == 0x8F) {
-		return unichar << 16
+	} else if (text_encoding_tag == 'J' && unichar == 0x8F) {
+		unichar = (unichar << 16)
 			| (readchar_nokeymap () << 8)
 			| readchar_nokeymap ();
 	} else {
 		byte2 = readchar_nokeymap ();
-		if (cjk_encoding == 'G' && '0' <= byte2 && byte2 <= '9') {
-			return unichar << 24
+		if (text_encoding_tag == 'G' && '0' <= byte2 && byte2 <= '9') {
+			unichar = (unichar << 24)
 				| (byte2 << 16)
 				| (readchar_nokeymap () << 8)
 				| readchar_nokeymap ();
 		} else {
-			return unichar << 8 | byte2;
+			unichar = (unichar << 8) | byte2;
+		}
+	}
+	if (return_unicode || utf8_text) {
+		unichar = lookup_encodedchar (unichar);
+		if (no_unichar (unichar)) {
+			return CHAR_INVALID;
 		}
 	}
-  } else {
 	return unichar;
+  } else {
+	if (mapped_term) {
+		if (! return_unicode && (cjk_text || mapped_text) && ! remap_chars ()) {
+#ifdef debug_mapped_keyboard
+			printf ("rc_m mapped (ret_u %d) -> %04X\n", return_unicode, unichar);
+#endif
+			return unichar;
+		} else {
+#ifdef debug_mapped_keyboard
+			printf ("rc_m mapped (ret_u %d) -> %04X -> %04X ...\n", return_unicode, unichar, lookup_mappedtermchar (unichar));
+#endif
+			unichar = lookup_mappedtermchar (unichar);
+			if (no_unichar (unichar)) {
+#ifdef debug_mapped_keyboard
+				printf ("rc_m mapped -> CHAR_INVALID\n");
+#endif
+				return CHAR_INVALID;
+			}
+		}
+	}
+
+	if (! return_unicode && (cjk_text || mapped_text)) {
+#ifdef debug_mapped_keyboard
+		printf ("rc_m -> %04X -> %04X\n", unichar, encodedchar (unichar));
+#endif
+		return encodedchar (unichar);
+	} else {
+#ifdef debug_mapped_keyboard
+		printf ("rc_m -> %04X\n", unichar);
+#endif
+		return unichar;
+	}
   }
 }
 
 /*
-   readcharacter () reads one Unicode character
+   readcharacter () reads one character
    keyboard mapping is suppressed
  */
 unsigned long
 readcharacter ()
 {
-  return readcharacter_mapping (False);
+  return readcharacter_mapping (False, False);
 }
 
 /*
-   readcharacter_mapped () reads one Unicode character
-   keyboard mapping is involved
+   readcharacter_mapped () reads one character
+   keyboard mapping is enabled
  */
 unsigned long
 readcharacter_mapped ()
 {
-  return readcharacter_mapping (True);
+  return readcharacter_mapping (True, False);
 }
 
+/*
+   readcharacter_unicode () reads one Unicode character
+   keyboard mapping is suppressed
+ */
+unsigned long
+readcharacter_unicode ()
+{
+  return readcharacter_mapping (False, True);
+}
+
+/*
+   readcharacter_unicode_mapped () reads one Unicode character
+   keyboard mapping is enabled
+ */
+unsigned long
+readcharacter_unicode_mapped ()
+{
+  return readcharacter_mapping (True, True);
+}
+
+
 /*-------------------------------------------------------------------------*/
 
 /*
@@ -624,13 +758,13 @@ DIRECTxtermgetxy (code)
 	    || is_menu_open ())
 	{
 		/* abort hilite tracking */
-		putstring ("\033[0;0;0;0;0T");
+		putescape ("\033[0;0;0;0;0T");
 		flush ();
 	} else {
 		build_string (track, "\033[1;%d;%d;%d;%dT", 
 				mouse_xpos + 1, mouse_ypos + 1 + MENU, 
 				1 + MENU, last_y + 3);
-		putstring (track);
+		putescape (track);
 		flush ();
 	}
   }
@@ -836,7 +970,7 @@ CURSgetxy ()
 static character queue [MAXCODELEN + 1], * endp = queue;
 static character ctrl_queue [MAXCODELEN + 1], * endcp = ctrl_queue;
 
-#ifdef unused
+#ifdef not_used
 static
 int
 q_empty ()
@@ -967,13 +1101,8 @@ findkeyin (str, fkeymap, matchmode, found)
   do {
 	char * strpoi = str;
 	char * mappoi = fkeymap [i].fk;
-	do {	/* compare str with map entry, ignore ";[0-9]" after ESC [ */
-		if (* strpoi == ';' && str [1] == '[') {
-			strpoi ++;
-			if (* strpoi >= '0' && * strpoi <= '9') {
-				strpoi ++;
-			}
-		}
+	do {
+		/* compare str with map entry */
 		if (* strpoi != * mappoi) {
 			break;
 		}
@@ -992,6 +1121,10 @@ findkeyin (str, fkeymap, matchmode, found)
 			/* str is equal to current entry */
 			* found = i;
 			keyproc = fkeymap [i].fp;
+			if (! keyproc) {
+				keyproc = I;
+			}
+			fkeyshift = fkeymap [i].fkeyshift;
 			if (matchmode == 1) {
 				/* return index unless other prefix 
 				   was or will be found */
@@ -1023,6 +1156,20 @@ findkeyin (str, fkeymap, matchmode, found)
   return ret;
 }
 
+static
+int
+findkey (str)
+  char * str;
+{
+  int dummy;
+  int res = findkeyin (str, fkeymap2, 0, & dummy);
+  if (res == -2) {
+	return findkeyin (str, fkeymap, 0, & dummy);
+  } else {
+	return res;
+  }
+}
+
 /*
    Look up key sequence in keyboard mapping table.
    Similar to findkeyin but with modified parameter details.
@@ -1123,6 +1270,9 @@ mapkb (str, kbmap, matchmode, found, mapped)
 			/* str is equal to current entry */
 			* found = kbmap [i].fk;
 			* mapped = kbmap [i].fp;
+			if (! * mapped) {
+				* mapped = I;
+			}
 			if (matchmode == 1) {
 				/* return index unless other prefix 
 				   was or will be found */
@@ -1156,20 +1306,6 @@ mapkb (str, kbmap, matchmode, found, mapped)
 #endif
 }
 
-static
-int
-findkey (str)
-  char * str;
-{
-  int dummy;
-  int res = findkeyin (str, fkeymap2, 0, & dummy);
-  if (res == -2) {
-	return findkeyin (str, fkeymap, 0, & dummy);
-  } else {
-	return res;
-  }
-}
-
 #endif	/* ifndef msdos */
 
 
@@ -1194,14 +1330,15 @@ showfkey ()
 int
 curs_readchar ()
 {
-  int keycode;
+  unsigned long keycode;
   keycode = __readchar ();
   while (keycode >= KEY_MIN) {
 	switch (keycode) {
 #ifdef KEY_RESIZE
 	case KEY_RESIZE:
   printf ("curses resize key received \n"); sleep (1);
-			keyproc = showfkey; return FUNcmd;
+			keyproc = showfkey;
+			return FUNcmd;
 #endif
 #ifdef KEY_SHIFT_L
 	case KEY_SHIFT_L:
@@ -1226,24 +1363,54 @@ curs_readchar ()
 struct {
 	chtype keycode;
 	voidfunc func;
+	unsigned char fkeyshift;
 } cursfunc [] =
 {
+	{KEY_F(1), F1},
+	{KEY_F(2), F2},
+	{KEY_F(3), F3},
+	{KEY_F(4), F4},
+	{KEY_F(5), F5},
+	{KEY_F(6), F6},
+	{KEY_F(7), F7},
+	{KEY_F(8), F8},
+	{KEY_F(9), F9},
+	{KEY_F(10), F10},
+	{KEY_F(11), F11},
+	{KEY_F(12), F12},
+	{KEY_IC, INSkey},
+	{KEY_DC, DELkey},
+	{KEY_HOME, HOMEkey},
+	{KEY_END, ENDkey},
+	{KEY_PPAGE, PU},
+	{KEY_NPAGE, PD},
+	{KEY_UP, MUP},
+	{KEY_DOWN, MDN},
+	{KEY_LEFT, MLF},
+	{KEY_RIGHT, MRT},
+#ifdef KEY_A2
+	{KEY_A2, MUP},
+#endif
+#ifdef KEY_C2
+	{KEY_C2, MDN},
+#endif
+#ifdef KEY_B1
+	{KEY_B1, MLF},
+#endif
+#ifdef KEY_B3
+	{KEY_B3, MRT},
+#endif
+	{KEY_B2, HOP},
+	{KEY_A1, HOMEkey},
+	{KEY_A3, PU},
+	{KEY_C1, ENDkey},
+	{KEY_C3, PD},
+	{0x1BF, BLINE},
+	{0x1C0, ELINE},
+
 #ifdef PAD0
 	{PAD0, PASTE},
 #endif
-	{KEY_F(1), HELP},
-	{KEY_F(2), WT},
-	{KEY_F(3), EDIT},
-	{KEY_F(4), INSFILE},
-	{KEY_F(5), insert_diaeresis},
-	{KEY_F(6), insert_diaeresis},
-	{KEY_F(7), SFW},
-	{KEY_F(8), SFW},
-	{KEY_F(9), RS},
-	{KEY_F(10), FILEMENU},
-	{KEY_F(20), QUICKMENU},
-	{KEY_F(11), LOWCAP},
-	{KEY_F(12), (voidfunc) UML},
 #ifdef __PDCURSES__
 	{PADSLASH, I},
 	{PADSTAR, I},
@@ -1254,17 +1421,9 @@ struct {
 	{ALT_F, FILEMENU},
 	{ALT_E, EDITMENU},
 	{ALT_S, SEARCHMENU},
-	{ALT_X, EXTRAMENU},
 	{ALT_P, PARAMENU},
+	{ALT_O, OPTIONSMENU},
 #endif
-	{KEY_IC, PASTE},
-	{KEY_DC, DCC},
-	{KEY_HOME, MARK},
-	{KEY_END, COPY},
-	{0x1BF, BLINE},
-	{0x1C0, ELINE},
-	{KEY_PPAGE, PU},
-	{KEY_NPAGE, PD},
 
 	{0}
 };
@@ -1288,6 +1447,7 @@ lookup_curskey (keycode)
   if (cursfunc [i].keycode == 0) {
 	return showfkey;
   } else {
+	keyshift |= cursfunc [i].fkeyshift;
 	return cursfunc [i].func;
   }
 }
@@ -1296,28 +1456,24 @@ lookup_curskey (keycode)
 
 
 /*
- * Read a character from terminal, considering function keys and 
- * composing special character of an 8 bit character set.
- * _readchar () takes the following actions:
- *  -	function key sequences according to the table fkeymap are 
- *	transformed into a special controlling character which is 
- *	assigned the function FUNKEY. Also the intended editor function, 
- *	as taken from the table fkeymap, is saved in a variable for 
- *	use by FUNKEY.
- *  -	the prefix keys for diacritic and special characters are 
- *	combined with the following key to make up the character.
+ * Functions to read a character from terminal.
+   If _readchar () detects a function key sequence (according to the 
+   table fkeymap), FUNcmd is returned, and the associated function is 
+   stored in the variable keyproc.
+   Keyboard mapping is applied.
  */
-int
+
+unsigned long
 _readchar_nokeymap ()
 {
-  int res;
+  unsigned long c;
   int prev_dont_suppress_keymap = dont_suppress_keymap;
 
   dont_suppress_keymap = 0;
-  res = _readchar ();
+  c = _readchar ();
   dont_suppress_keymap = prev_dont_suppress_keymap;
 
-  return res;
+  return c;
 }
 
 #ifndef msdos
@@ -1382,7 +1538,11 @@ read1byte ()
 #ifdef CURSES
 	return curs_readchar ();
 #else
-	return __readchar ();
+	if (report_winchg) {
+		return __readchar_report_winchg ();
+	} else {
+		return __readchar ();
+	}
 #endif
   }
 }
@@ -1481,13 +1641,14 @@ static menuitemtype radicalsmenu [] =
 #endif
 
 
-int
+#define dont_debug_kbesc
+#define dont_debug_kbmap
+
+static
+unsigned long
 _readchar ()
 {
-  register character ch;
-#ifdef dosmouse
-  int ich;
-#endif
+  unsigned long ch;
 #ifndef msdos
   int res;
   char * found;
@@ -1523,7 +1684,7 @@ _readchar ()
   queue_mode_mapped = False;
 #endif
 
-  keyshift = '0';
+  keyshift = 0;
 
 
 #ifdef radical_stroke
@@ -1558,7 +1719,7 @@ _readchar ()
 	while (choice_index < 0) {
 		int radical = popup_menu (radicalsmenu, arrlen (radicalsmenu), 
 					choice_col, choice_line, 
-					"Select radical", False, "1234567890x");
+					"Select radical", False, False, "1234567890x");
 		if (radical < 0) {
 			/* revert to previous mapping (or none) */
 			toggleKEYMAP ();
@@ -1601,90 +1762,71 @@ _readchar ()
 #ifdef CURSES
 
   key = read1byte ();
-  while (key >= KEY_MIN) {
-	switch (key) {
-#ifdef KEY_A2
-	case KEY_A2:
-#endif
-	case KEY_UP:	keyproc = MUP; return FUNcmd;
-#ifdef KEY_C2
-	case KEY_C2:
-#endif
-	case KEY_DOWN:	keyproc = MDN; return FUNcmd;
-#ifdef KEY_B1
-	case KEY_B1:
-#endif
-	case KEY_LEFT:	keyproc = MLF; return FUNcmd;
-#ifdef KEY_B3
-	case KEY_B3:
-#endif
-	case KEY_RIGHT:	keyproc = MRT; return FUNcmd;
-	case KEY_B2:	keyproc = HOP; return FUNcmd;
-	case KEY_A1:	keyproc = MARK; return FUNcmd;
-	case KEY_A3:	keyproc = PU; return FUNcmd;
-	case KEY_C1:	keyproc = COPY; return FUNcmd;
-	case KEY_C3:	keyproc = PD; return FUNcmd;
+  if (key >= KEY_MIN) {
 #ifdef KEY_MOUSE
-	case KEY_MOUSE:
-			CURSgetxy ();
-			keyproc = MOUSEescape;
-			return FUNcmd;
-#endif
-
-	default:
-		keyproc = lookup_curskey (key); return FUNcmd;
+	if (key == KEY_MOUSE) {
+		CURSgetxy ();
+		keyproc = MOUSEescape;
+		return FUNcmd;
 	}
-	/* loop should not occur, handled in curs_readchar */
-	key = read1byte ();
+#endif
+	keyproc = lookup_curskey (key);
+	return FUNcmd;
   }
   ch = key;
 
 #else	/* ifdef CURSES */
 
 #ifdef dosmouse
-  ich = __readchar ();
-  if (ich < 0) {
-	DIRECTdosmousegetxy ();
-	keyproc = MOUSEescape;
-	return FUNcmd /* index of FUNKEY */;
-  } else {
-	ch = ich;
+  {
+	int ich = __readchar ();
+	if (ich < 0) {
+		DIRECTdosmousegetxy ();
+		keyproc = MOUSEescape;
+		return FUNcmd;
+	} else {
+		ch = ich;
+	}
   }
 #else
   ch = read1byte ();
+  if (ch == FUNcmd) {
+	/* pass back WINCH exception */
+	return FUNcmd;
+  }
 #endif
 
 #endif	/* else CURSES */
 
 
 #ifdef msdos
+
   if (ch == '\000') {
 # ifdef dosmouse
 	ch = getch ();
 # else
 	ch = read1byte ();
 # endif
-	keyproc = pc_fkey_map [ch];
-# ifdef DEBUG
+	keyproc = pc_fkey_map [ch].fp;
+	if (! keyproc) {
+		keyproc = I;
+	}
+	keyshift = pc_fkey_map [ch].fkeyshift;
+# ifdef debug_whatever
 	if ((voidfunc) keyproc == (voidfunc) I) {
 		return ch;
 	}
 	/* (voidfunc) is an identity cast here. It seems to be required 
 	   for the sake of the apparently totally rotten microvax compiler */
 # endif
-	return FUNcmd /* index of FUNKEY */;
-  }
-#else
-  if (ch == '\000') {
-	keyproc = MARK;
 	return FUNcmd;
+  } else {
+	return ch;
   }
-#endif
 
+#else
 
-#ifndef msdos
-
-    else if (ch == '\033') {
+  if (ch == '\033') {
 	char second_esc_char = '\0';
 
 	q_put (ch);
@@ -1696,41 +1838,168 @@ _readchar ()
 	{
 		char prev_ch = ch;
 		ch = read1byte ();
+#ifdef debug_kbesc
+		printf ("<^[%s> + %02X\n", ctrl_queue + 1, ch);
+#endif
 
+		/* accept ESC prefix as Alt modifier */
+		if (detect_esc_alt && ch == '\033' && prev_ch == '\033') {
+			if (bytereadyafter (escape_delay)) {
+				ch = read1byte ();
+				keyshift |= alt_mask;
+			}
+		}
+
+		/* accept XTerm*modifyCursorKeys: 3 as well as 
+		   device attributes reports (primary/secondary DA) */
+		if (second_esc_char == '[' && prev_ch == '[' &&
+			(ch == '>' || ch == '?')
+		   ) {
+			if (ch == '?') {
+				keyshift |= report_mask;
+			}
+			ch = read1byte ();
+		}
+
+check_ctrl_byte:
 		/* handle generic shift state of ANSI sequences: */
 		if (ch == ';' && second_esc_char == '[') {
-			/* ESC [1;2A -> ESC [1A + shift etc */
-			q_put (';');
 			ch = read1byte ();
-			keyshift = ch - 1;
-			q_put (ch);
+			if (ch >= '2' && ch <= '8') {
+				/* ESC [1;2A -> ESC [1A + shift etc */
+				keyshift |= (ch & ~'0') - 1;
+			} else if (ch == '1') {
+				ch = read1byte ();
+				if (ch >= '0' && ch <= '6') {
+					/* xterm with alwaysUseMods: */
+					/* ESC [1;11A -> ESC [1A + alt etc */
+					keyshift |= applkeypad_mask | ((ch & ~'0')  + 1);
+				} else {
+					q_put (';');
+					q_put ('1');
+					q_put (ch);
+				}
+			} else {
+				q_put (';');
+				q_put (ch);
+			}
 			ch = read1byte ();
-		} else if (ch >= '0' && ch <= '9' && second_esc_char == 'O') {
-			/* ESC O2A -> ESC OA + shift etc */
-			keyshift = ch - 1;
+		} else if (ch == ';' && second_esc_char == '1') {
 			ch = read1byte ();
+			if (ch >= '2' && ch <= '8') {
+				/* xterm/VT52 mode: ESC 1;2A -> ESC 1A + shift etc */
+				keyshift |= (ch & ~'0')  - 1;
+			} else {
+				q_put (';');
+				q_put (ch);
+			}
+			ch = read1byte ();
+		} else if (ch >= '0' && ch <= '9'
+			   && (second_esc_char == 'O' || 
+			       (hp_shift_mode && ! second_esc_char))) {
+			if (ch >= '2' && ch <= '8') {
+				/* ESC O2A -> ESC OA + shift etc */
+				/* HP: ESC 2p -> ESC p + shift etc */
+				keyshift |= (ch & ~'0')  - 1;
+			} else if (ch == '1') {
+				ch = read1byte ();
+				if (ch >= '0' && ch <= '6') {
+					/* ESC O11P -> ESC OP + alt etc */
+					/* HP: ESC 11p -> ESC p + alt etc */
+					keyshift |= applkeypad_mask | ((ch & ~'0')  + 1);
+				} else {
+					q_put ('1');
+					q_put (ch);
+				}
+			} else {
+				q_put (ch);
+			}
+			ch = read1byte ();
+		} else if (sco_shift_mode && second_esc_char == '['
+			   && ch >= '0' && ch <= '9') {
+			char newkeyshift;
+			char newch = '\0';
+			if (ch >= '2' && ch <= '8') {
+				/* SCO: ESC [2M -> ESC [M + shift etc */
+				newkeyshift = (ch & ~'0')  - 1;
+				newch = read1byte ();
+#ifdef debug_kbesc
+		printf ("-> <^[%s> + 2-8/%02X\n", ctrl_queue + 1, newch);
+#endif
+				if (newch >= 'A' && newch <= 'Z') {
+					keyshift |= newkeyshift;
+				} else {
+					q_put (ch);
+				}
+			} else if (ch == '1') {
+				ch = read1byte ();
+#ifdef debug_kbesc
+		printf ("-> <^[%s> + 1/%02X\n", ctrl_queue + 1, ch);
+#endif
+				if (ch >= '0' && ch <= '6') {
+					/* SCO: ESC [11M -> ESC [M + alt etc */
+					newkeyshift = applkeypad_mask | ((ch & ~'0')  + 1);
+					newch = read1byte ();
+					if (newch >= 'A' && newch <= 'Z') {
+						keyshift |= newkeyshift;
+					} else {
+						q_put ('1');
+						q_put (ch);
+					}
+				} else if (ch == ';') {
+					/* xterm SCO: ESC [1;5R -> ESC [R + alt etc */
+					ch = read1byte ();
+					keyshift |= (ch & ~'0') - 1;
+					newch = read1byte ();
+				} else {
+					q_put ('1');
+					q_put (ch);
+					newch = read1byte ();
+				}
+			} else {
+#ifdef debug_kbesc
+		printf ("-> <^[%s>\n", ctrl_queue + 1);
+#endif
+				q_put (ch);
+				newch = read1byte ();
+			}
+			ch = newch;
+			if (ch == ';') {
+				/* OK, so here is my second goto; 
+				   this is really a very peculiar case, 
+				   based on misconfiguration 
+				   (mined assumes sco, terminal sends 
+				   modified xterm sequences), 
+				   which does not deserve any 
+				   goto avoidance effort
+				*/
+#ifdef debug_kbesc
+				printf ("GOTO check_ctrl_byte\n");
+#endif
+				goto check_ctrl_byte;
+			}
 		} else if (prev_ch >= '0' && prev_ch <= '9') {
 			if (ch == '^') {
 				/* ESC [5^ -> ESC [5~ + control (rxvt) */
-				keyshift = ctrl_mask;
+				keyshift |= ctrl_mask;
 				ch = '~';
 			} else if (ch == '$') {
 				/* ESC [5$ -> ESC [5~ + shift (rxvt) */
-				keyshift = shift_mask;
+				keyshift |= shift_mask;
 				ch = '~';
 			} else if (ch == '@') {
 				/* ESC [5@ -> ESC [5~ + control+shift (rxvt) */
-				keyshift = ctrlshift_mask;
+				keyshift |= ctrlshift_mask;
 				ch = '~';
 			}
 		} else if (ch >= 'a' && ch <= 'd') {
 			if (prev_ch == 'O') {
 				/* ESC Oa -> ESC OA + control (rxvt) */
-				keyshift = ctrl_mask;
+				keyshift |= ctrl_mask;
 				ch -= 'a' - 'A';
 			} else if (prev_ch == '[') {
 				/* ESC [a -> ESC [A + shift (rxvt) */
-				keyshift = shift_mask;
+				keyshift |= shift_mask;
 				ch -= 'a' - 'A';
 			}
 		}
@@ -1738,17 +2007,25 @@ _readchar ()
 		q_put (ch);
 		if (second_esc_char == '\0') {
 			second_esc_char = ch;
+			if (second_esc_char == 'O') {
+				keyshift |= applkeypad_mask;
+			}
 		}
 	}
+#ifdef debug_kbesc
+	printf ("=> <^[%s> [%X]\n", ctrl_queue + 1, keyshift);
+#endif
 
 	if (quit) {
 		keyproc = I;
 		return FUNcmd;
-	} else if (res < 0) { /* key pattern not detected in fkeymap table */
+	} else if (res < 0) { /* key pattern not found in fkeymap table */
 		if (second_esc_char == '[') {
 			/* generic scanning for unknown ANSI sequence */
 			(void) q_get (); /* swallow ESC */
 			(void) q_get (); /* swallow [ */
+			putback_rest (ctrl_queue + 2);
+
 			ansi_params = 0;
 			do {
 				int i;
@@ -1758,16 +2035,122 @@ _readchar ()
 					ansi_params ++;
 				}
 			} while (ansi_fini == ';');
+			q_clear ();
 
-			keyproc = ANSIseq;
-			return FUNcmd;
+			if (ansi_params >= 3
+			    && ansi_param [0] == 27
+			    && ansi_fini == '~') {
+				/* handle generic escape sequence 
+				   for modified key 
+				   (xterm 214 resource modifyOtherKeys)
+				 */
+				if (ansi_param [1] > 8) {
+					ansi_param [1] -= 8;
+				}
+				keyshift |= ansi_param [1] - 1;
+
+				/* enforce UTF-8 interpretation of input */
+				queue_mode_mapped = True;
+
+				if (ansi_param [2] == ' ' && (keyshift & ctrlshift_mask) == ctrl_mask) {
+					/* map ctrl-space to MARK,
+					   alt-ctrl-space to Hop MARK */
+					q_put ('\0');
+				} else if (ansi_param [2] == 0) {
+					q_put ('\0');
+				} else {
+					char cbuf [7];
+					char * cp = cbuf;
+
+					/* Shift-a/A -> A */
+					if ((keyshift & altctrlshift_mask) == shift_mask
+					   && isLetter (ansi_param [2])
+					   ) {
+						keyshift = 0;
+						ansi_param [2] = case_convert (ansi_param [2], 1);
+					}
+
+					/* Control-/ etc as accent prefix */
+					if (keyshift & ctrl_mask) {
+						struct prefixspec * ps;
+						ps = lookup_prefix (COMPOSE, ansi_param [2]);
+						if (ps && * ps->accentname) {
+							keyshift = ansi_param [2];
+							keyproc = COMPOSE;
+							return FUNcmd;
+						} else if (ps) {
+							unsigned long unichar;
+							int len;
+							utf8_info (ps->pat1, & len, & unichar);
+							ansi_param [2] = unichar;
+						} else if ('?' <= ansi_param [2] && ansi_param [2] <= '') {
+							keyshift &= ~ ctrl_mask;
+							return ansi_param [2] & 0x1F;
+						}
+					}
+
+					(void) utfencode (ansi_param [2], cbuf);
+					while (* cp) {
+						q_put (* cp ++);
+					}
+				}
+
+				/* Control/Alt/Control-Alt-0 -> ...key_0 */
+				if (ansi_param [2] >= '0' && ansi_param [2] <= '9') {
+					switch (ansi_param [2]) {
+					case '0': keyproc = key_0; break;
+					case '1': keyproc = key_1; break;
+					case '2': keyproc = key_2; break;
+					case '3': keyproc = key_3; break;
+					case '4': keyproc = key_4; break;
+					case '5': keyproc = key_5; break;
+					case '6': keyproc = key_6; break;
+					case '7': keyproc = key_7; break;
+					case '8': keyproc = key_8; break;
+					case '9': keyproc = key_9; break;
+					}
+					q_clear ();
+					return FUNcmd;
+				}
+
+				if (keyshift & alt_mask) {
+					/* 2000.13 workaround for prompt */
+					if ((keyshift & altctrl_mask) == alt_mask) {
+					    if (ansi_param [2] == 'k') {
+						q_clear ();
+						keyproc = toggleKEYMAP;
+						return FUNcmd;
+					    }
+					    if (ansi_param [2] == 'K'
+					     || ansi_param [2] == 'I') {
+						q_clear ();
+						keyproc = setupKEYMAP;
+						return FUNcmd;
+					    }
+					}
+
+					return '\033';
+				} else if (keyshift) {
+					return q_get ();
+				} else {
+					/* fall through to keyboard mapping */
+				}
+			} else {	/* not CSI 27 sequence */
+				keyproc = ANSIseq;
+				return FUNcmd;
+			}
+		} else if (keyshift & alt_mask) {
+			/* restore the previously filtered ESCAPE */
+			return '\033';
 		} else {
 			/* just deliver the typed characters */
 			return q_get ();
 		}
-	} else {
+	} else {	/* key pattern found in fkeymap table */
 		q_clear ();
 
+		keyshift |= fkeyshift;
+
 		if (keyproc == DIRECTxterm) {
 			DIRECTxtermgetxy ('M');
 			keyproc = MOUSEescape;
@@ -1782,13 +2165,14 @@ _readchar ()
 			keyproc = MOUSEescape;
 		}
 
-		return FUNcmd /* index of FUNKEY */;
+		return FUNcmd;
 	}
-  }
-#define dont_debug_kbmap
-    else if (allow_keymap && dont_suppress_keymap
-		&& keyboard_map != (keymaptabletype) NIL_PTR) {
+  } else {
 	q_put (ch);
+  }
+
+  if (allow_keymap && dont_suppress_keymap
+		&& keyboard_map != (keymaptabletype) NIL_PTR) {
 	while  ((res = mapkb (ctrl_queue, keyboard_map, 1, & found, & mapped)) == -1
 		 /* prefix of some table entry */
 		&& q_notfull ()
@@ -1815,21 +2199,23 @@ _readchar ()
 		res = 1;
 		if (found == NIL_PTR) {
 #ifdef debug_kbmap
-	printf ("rest %s, ", ctrl_queue + strlen (prev_found));
+			printf ("rest %s, ", ctrl_queue + strlen (prev_found));
 #endif
 			putback_rest (ctrl_queue + strlen (prev_found));
 		}
 	} else if (res == -1) {
 		res = mapkb (ctrl_queue, keyboard_map, 2, & found, & mapped);
 #ifdef debug_kbmap
-	printf ("find again %s, found %s, ", ctrl_queue, found);
+		printf ("find again %s, found %s, ", ctrl_queue, found);
 #endif
 	}
 
 #ifdef debug_kbmap
 	printf ("res %d\n", res);
 #endif
+
 	if (res >= 0) { /* key pattern detected in key mapping table */
+
 handle_mapped:
 		q_clear ();
 
@@ -1844,7 +2230,7 @@ handle_mapped:
 		if (choice != NIL_PTR && * choice != '\0') {
 			choice_index = choose_char (mapped);
 #ifdef debug_kbmap
-	printf (" choose_char (%s) -> %d\n", mapped, choice_index);
+			printf (" choose_char (%s) -> %d\n", mapped, choice_index);
 #endif
 			if (choice_index < 0) {
 				mapped = NIL_PTR;
@@ -1873,7 +2259,7 @@ handle_mapped:
 			}
 		}
 #ifdef debug_kbmap
-	printf (" queued %s\n", ctrl_queue);
+		printf (" queued %s\n", ctrl_queue);
 #endif
 	}
 
@@ -1884,15 +2270,14 @@ handle_mapped:
 		flush ();	/* clear key mapping menu */
 		return _readchar ();
 	}
+  } else {	/* not keyboard mapped */
+	return q_get ();
   }
 
-#endif	/* #ifndef msdos */
-
-    else {
-	return ch;
-  }
+#endif	/* #ifdef msdos #else */
 }
 
+
 #ifndef msdos
 
 #define dont_debug_choose_char
@@ -1992,11 +2377,10 @@ choose_char (choices)
 		* choicelinepoi ++ = '\0';
 		/* add menu line to menu */
 #ifdef debug_choose_char
-	printf ("choiceline <%s>\n", thischoiceline);
+		printf ("choiceline <%s>\n", thischoiceline);
 #endif
-		choicemenu [li].itemname = thischoiceline;
-		choicemenu [li].itemfu = dummyfunc;
-		choicemenu [li].hopitemname = "";
+		/* fill menu item with collected data */
+		fill_menuitem (& choicemenu [li], thischoiceline);
 
 		thischoiceline = choicelinepoi;
 	}
@@ -2017,7 +2401,7 @@ choose_char (choices)
 #endif
 	selected = popup_menu (choicemenu, lastrow + 1, 
 				choice_col, choice_line, 
-				"Choose character", False, "1234567890");
+				"Choose character", False, False, "1234567890");
 	if (in_status_line) {
 		redraw_prompt ();
 #ifdef cut_picklist_to_screen
