@@ -11,7 +11,13 @@
 \*======================================================================*/
 
 #ifdef use_keymap_tables
+
+#ifdef keymapping_stringtables
 #include "_keymaps.h"
+#else
+#include "keymapsi.h"
+#endif
+
 #endif
 
 
