@@ -1,7 +1,10 @@
 	{keymap_GreekMonotonic, "gr"},
 	{keymap_Cyrillic, "cy"},
+	{keymap_Kazakh_prefix, "kz"},
 	{keymap_Hebrew, "he"},
 	{keymap_ArabicKeyboard, "ar"},
+	{keymap_Amharic, "am"},
+	{keymap_Ethiopic, "et"},
 	{keymap_Pinyin, "py"},
 	{keymap_Cangjie, "cj"},
 	{keymap_WuBi, "wu"},
@@ -15,4 +18,5 @@
 	{keymap_HANJA, "hj"},
 	{keymap_VNI, "vi"},
 	{keymap_VIQR, "vq"},
-	{keymap_Thai, "th"},
+	{keymap_th_kesmanee, "th"},
+	{keymap_Fullwidth, "fw"},
