@@ -16,6 +16,7 @@ FLAG input_active = False;
 \*======================================================================*/
 
 static char * lastinbuf;
+static FLAG lastattrib;
 static char status_buf [maxLINE_LEN];
 static char * mnemo = NIL_PTR;
 
@@ -41,12 +42,12 @@ print_mark ()
 	reverse_off ();
 	dim_on ();
 	if (cjk_term) {
-		put_cjkchar (cjk (0x300A));	/* 《 */
+		put_cjkchar (encodedchar (0x300A));	/* 《 */
 		if (cjk_lineend_width == 2) {
 			lpos ++;
 		}
 	} else {
-		put_unichar ((character) RET_MARK);
+		put_unichar ((character) RET_marker);
 	}
 	lpos ++;
 	dim_off ();
@@ -89,7 +90,7 @@ print_byte (c)
 /**
    Print a mapped character to status line.
    Account for column position and possible line-wrap.
-   Only called if (mapped_text && combining_mode == False && encoding_has_combining ())
+   Only called if (mapped_text && ! combining_mode && encoding_has_combining ())
  */
 static
 void
@@ -99,7 +100,7 @@ print_mapped (c, char_begin, line_begin)
   char * line_begin;
 {
   int width;
-  unsigned long unichar = lookup_cjk (c);
+  unsigned long unichar = lookup_encodedchar (c);
 
   char_on_status_line = True;
 
@@ -129,7 +130,7 @@ print_mapped (c, char_begin, line_begin)
 	put_unichar (controlchar (c));
 	lpos ++;
   } else {
-	if (combining_mode == False && combining_screen && iscombined (unichar, char_begin, line_begin)) {
+	if (! combining_mode && combining_screen && iscombined (unichar, char_begin, line_begin)) {
 		/* enforce separated display */
 		putcharacter (' ');
 		putcharacter (c);
@@ -156,44 +157,44 @@ print_cjk (c, width, char_begin, line_begin)
   char * char_begin;
   char * line_begin;
 {
-  unsigned long unichar = lookup_cjk (c);
+  unsigned long unichar = lookup_encodedchar (c);
 
-	if (encoding_has_combining () && iscombined (unichar, char_begin, line_begin)) {
-		if (combining_mode == False && combining_screen) {
+  if (encoding_has_combining () && iscombined (unichar, char_begin, line_begin)) {
+	if (! combining_mode && combining_screen) {
 		/* enforce separated display */
-		/*	disp_separate_combining = True;	*/
-		/*	combiningdisp_on ();		*/
+	/*	disp_separate_combining = True;	*/
+	/*	combiningdisp_on ();		*/
 
-			if (width == 2) {
-				put_cjkchar (cjk (0x3000));
-				put_cjkchar (c);
-			} else {
-				if (isjoined (unichar, char_begin, line_begin)) {
-					/* prevent terminal ligature by 
-					   substituting joining character 
-					   with its ISOLATED FORM */
-					unichar = isolated_alef (unichar);
-					c = cjk (unichar);
-					if (no_char (c)) {
-						put_cjkchar (cjk (0x2135));
-					} else {
-						put_cjkchar (c);
-					}
-				} else {
-					put_cjkchar (' ');
-					put_cjkchar (c);
-				}
-			}
-		} else if (combining_screen == False) {
-		/*	disp_separate_combining = True;	*/
-		/*	combiningdisp_on ();		*/
+		if (width == 2) {
+			put_cjkchar (encodedchar (0x3000));
 			put_cjkchar (c);
 		} else {
-			put_cjkchar (c);
+			if (isjoined (unichar, char_begin, line_begin)) {
+				/* prevent terminal ligature by 
+				   substituting joining character 
+				   with its ISOLATED FORM */
+				unichar = isolated_alef (unichar);
+				c = encodedchar (unichar);
+				if (no_char (c)) {
+					put_cjkchar (encodedchar (0x2135));
+				} else {
+					put_cjkchar (c);
+				}
+			} else {
+				put_cjkchar (' ');
+				put_cjkchar (c);
+			}
 		}
+	} else if (! combining_screen) {
+	/*	disp_separate_combining = True;	*/
+	/*	combiningdisp_on ();		*/
+		put_cjkchar (c);
 	} else {
 		put_cjkchar (c);
 	}
+  } else {
+	put_cjkchar (c);
+  }
 }
 
 /**
@@ -224,7 +225,7 @@ print_unichar (unichar, width, char_begin, line_begin)
 	lpos += width;
 
 	if (iscombined (unichar, char_begin, line_begin)) {
-		if (combining_mode == False && combining_screen) {
+		if (! combining_mode && combining_screen) {
 		/* enforce separated display */
 		/*	disp_separate_combining = True;	*/
 		/*	combiningdisp_on ();		*/
@@ -243,7 +244,7 @@ print_unichar (unichar, width, char_begin, line_begin)
 					put_unichar (unichar);
 				}
 			}
-		} else if (combining_screen == False) {
+		} else if (! combining_screen) {
 		/*	disp_separate_combining = True;	*/
 		/*	combiningdisp_on ();		*/
 			put_unichar (unichar);
@@ -262,9 +263,10 @@ print_unichar (unichar, width, char_begin, line_begin)
  */
 static
 void
-printlim_string (text, limit)
+printlim_string (text, limit, attributes)
   char * text;
-  register int limit;
+  int limit;
+  FLAG attributes;
 {
   int utfcount;
   unsigned long unichar;
@@ -282,16 +284,20 @@ printlim_string (text, limit)
 		utf8_info (text, & utfcount, & unichar);
 		charwidth = uniscrwidth (unichar, text, start);
 	} else if (iscontrol (* text)) {
-		charwidth = 2;
-	} else if (cjk_text /* && multichar (* text) */) {
+		if (attributes && * text != '\n') {
+			charwidth = 0;
+		} else {
+			charwidth = 2;
+		}
+	} else if (cjk_text) {
 		cjkchar = charvalue (text);
-		charwidth = cjkscrwidth (unichar, text, start);
+		charwidth = cjkscrwidth (cjkchar, text, start);
 	} else {
 		charwidth = 1;
 	}
 
 	if (limit > 0 && lpos > limit - charwidth) {
-		putmark (SHIFT_MARK, NIL_PTR);
+		putmark (SHIFT_marker, UTF_SHIFT_marker);
 		break;
 	}
 	if (lpos > XBREAK_bottom - 1) {
@@ -305,13 +311,28 @@ printlim_string (text, limit)
 		print_mark ();
 		text ++;
 	} else if (iscontrol (* text)) {
-		lpos += 2;
-		putchar ('^');
-		put_unichar (controlchar (* text ++));
+		if (attributes) {
+			switch (* text) {
+			case '':	unidisp_on ();
+					break;
+			case '':	dim_on ();
+					menuheader_on ();
+					break;
+			case '':	disp_normal ();
+					break;
+			case '':	diagdisp_on ();
+					break;
+			}
+			text ++;
+		} else {
+			lpos += 2;
+			putchar ('^');
+			put_unichar (controlchar (* text ++));
+		}
 	} else if (utf8_text) {
 		print_unichar (unichar, charwidth, text, start);
 		text += utfcount;
-	} else if (cjk_text /* && multichar (* text) */) {
+	} else if (cjk_text) {
 		/*int prev_lpos = lpos;*/
 		advance_char_scr (& text, & lpos, start);
 		/*put_cjkchar (cjkchar, lpos - prev_lpos);*/
@@ -332,7 +353,7 @@ void
 print_string (text)
   register char * text;
 {
-  printlim_string (text, 0);
+  printlim_string (text, 0, False);
 }
 
 
@@ -347,7 +368,7 @@ character
 promptyn ()
 {
   register character c;
-  while ((c = readcharacter ()) != 'y' && c != 'n' && c != '\033' && quit == False) {
+  while ((c = readcharacter ()) != 'y' && c != 'n' && c != '\033' && ! quit) {
 	ring_bell ();
 	flush ();
   }
@@ -393,7 +414,7 @@ get_num_char (result, maxvalue)
 
   diagdisp_off ();
 
-  while ((maxfill < maxvalue) && quit == False) {
+  while (maxfill < maxvalue && ! quit) {
 	flush ();
 	c = readcharacter ();
 
@@ -444,7 +465,7 @@ get_num_char (result, maxvalue)
 
   diagdisp_on ();
   if (uniinput && (cjk_text || mapped_text)) {
-	* result = cjk (unichar);
+	* result = encodedchar (unichar);
   } else {
 	* result = unichar;
   }
@@ -463,7 +484,7 @@ input_byte (inbuf, ptr, c)
   character c;
 {
   if (c > '\0' && (ptr - inbuf) < maxLINE_LEN) {
-	if (mapped_text && combining_mode == False && encoding_has_combining ()) {
+	if (mapped_text && ! combining_mode && encoding_has_combining ()) {
 		print_mapped (c, ptr, inbuf);
 	} else {
 		print_byte (c);
@@ -476,14 +497,14 @@ input_byte (inbuf, ptr, c)
   }
 
   if (mapped_text && combining_mode && encoding_has_combining ()) {
-	if (iscombining (lookup_cjk (c))) {
+	if (iscombining (lookup_encodedchar (c))) {
 		/* refresh display from base char, needed on xterm */
 		unsigned long uc;
 		char * refreshptr = ptr;
 		do {
 			precede_char (& refreshptr, inbuf);
 			c = charvalue (refreshptr);
-			uc = lookup_cjk (c);
+			uc = lookup_encodedchar (c);
 		} while (refreshptr != inbuf && iscombining (uc));
 		if (! iscombining (uc)) {
 			putstring ("\b");
@@ -491,7 +512,7 @@ input_byte (inbuf, ptr, c)
 		/* Now refresh the whole combined character */
 		while (refreshptr < ptr) {
 			c = charvalue (refreshptr ++);
-			put_unichar (lookup_cjk (c));
+			put_unichar (lookup_encodedchar (c));
 		}
 	}
   }
@@ -542,14 +563,14 @@ input_cjk (inbuf, ptr, c)
 	}
 
 	if (combining_mode && encoding_has_combining ()) {
-		if (iscombining (lookup_cjk (c))) {
+		if (iscombining (lookup_encodedchar (c))) {
 			/* refresh display from base char, needed on xterm */
 			unsigned long uc;
 			char * refreshptr = ptr;
 			do {
 				precede_char (& refreshptr, inbuf);
 				c = charvalue (refreshptr);
-				uc = lookup_cjk (c);
+				uc = lookup_encodedchar (c);
 			} while (refreshptr != inbuf && iscombining (uc));
 			if (! iscombining (uc)) {
 				if (cjkscrwidth (c, refreshptr, inbuf) == 2) {
@@ -681,11 +702,13 @@ char *
 input_character (inbuf, ptr, c)
   char * inbuf;
   char * ptr;
-  long c;
+  unsigned long c;
 {
 	if (no_char (c)) {
 		ring_bell ();
 		return ptr;
+	} else if (c < 0x80) {
+		return input_byte (inbuf, ptr, c);
 	} else if (utf8_text) {
 		return input_unicode (inbuf, ptr, c);
 	} else if (cjk_text) {
@@ -701,11 +724,14 @@ input_character (inbuf, ptr, c)
 void
 redraw_prompt ()
 {
-	set_cursor (0, 0);
+	set_cursor (0, - MENU);
 	while (bottom_lines > 0) {
 		scroll_reverse ();
 		bottom_lines --;
 	}
+	if (MENU && ! can_add_line) {
+		displaymenuline ();
+	}
 	rd_bottom_line ();
 	if (can_clear_eol) {
 		clear_eol ();
@@ -714,10 +740,13 @@ redraw_prompt ()
 
 static
 char *
-erase1 (inbuf, ptr)
+erase1 (inbuf, ptr, all_combined)
   char * inbuf;
   char * ptr;
+  FLAG all_combined;
 {
+  FLAG need_refresh = False;
+
 	diagdisp_off ();
 	precede_char (& ptr, inbuf);
 	if (iscontrol (* ptr)) {
@@ -730,14 +759,19 @@ erase1 (inbuf, ptr)
 		lpos = lpos - 2;
 	    }
 	} else {
-	    unsigned long charval;
+	    unsigned long charval = charvalue (ptr);
 	    int charwidth;
 
 	    if (combining_mode && encoding_has_combining ()) {
-		charval = charvalue (ptr);
-		while (ptr != inbuf && iscombined (unicode (charval), ptr, inbuf)) {
-			precede_char (& ptr, inbuf);
-			charval = charvalue (ptr);
+		if (all_combined) {
+			while (ptr != inbuf && iscombined (unicode (charval), ptr, inbuf)) {
+				precede_char (& ptr, inbuf);
+				charval = charvalue (ptr);
+			}
+		} else if (iscombined (unicode (charval), ptr, inbuf)) {
+			/* work-around for refresh of base character 
+			   without or with fewer combining accents */
+			need_refresh = True;
 		}
 	    }
 
@@ -746,7 +780,7 @@ erase1 (inbuf, ptr)
 	    } else if (cjk_text) {
 		charwidth = cjkscrwidth (charval, ptr, inbuf);
 	    } else if (mapped_text && combining_mode && encoding_has_combining ()
-			&& iscombined (lookup_cjk (charval), ptr, inbuf)) {
+			&& iscombined (lookup_encodedchar (charval), ptr, inbuf)) {
 		charwidth = 0;
 	    } else {
 		charwidth = 1;
@@ -764,9 +798,74 @@ erase1 (inbuf, ptr)
 	putstring (" \b");
 	* ptr = '\0';
 
+	if (need_refresh) {
+		redraw_prompt ();
+	}
+
 	return ptr;
 }
 
+static
+char *
+input_prefix (inbuf, ptr, ps)
+  char * inbuf;
+  char * ptr;
+  struct prefixspec * ps;
+{
+  unsigned long c = readcharacter_unicode_mapped ();
+
+  if (command (c) == DPC) {
+	if (mnemo == NIL_PTR ? ptr > inbuf : (char *) ptr > mnemo) {
+		return erase1 (inbuf, ptr, False);
+	} else {
+		ring_bell ();
+		return ptr;
+	}
+  } else if ((command (c) == CTRLINS && (c = readcharacter_unicode_mapped ()) == FUNcmd && (keyshift |= ctrl_mask))
+	     || c == FUNcmd) {
+	struct prefixspec * ps2 = lookup_prefix (keyproc, keyshift);
+	if (ps2) {
+		c = readcharacter_unicode_mapped ();
+		if ((command (c) == CTRLINS && (c = readcharacter_unicode_mapped ()) == FUNcmd && (keyshift |= ctrl_mask))
+		    || c == FUNcmd) {
+			struct prefixspec * ps3 = lookup_prefix (keyproc, keyshift);
+			if (ps3) {
+				c = readcharacter_unicode_mapped ();
+				c = compose_patterns (c, ps, ps2, ps3);
+				return input_character (inbuf, ptr, c);
+			} else {
+				ring_bell ();
+				return ptr;
+			}
+		} else {
+			c = compose_patterns (c, ps, ps2, 0);
+			return input_character (inbuf, ptr, c);
+		}
+	} else {
+		ring_bell ();
+		return ptr;
+	}
+  } else {
+	c = compose_patterns (c, ps, 0, 0);
+	return input_character (inbuf, ptr, c);
+  }
+}
+
+static
+voidfunc
+command1 (c)
+  unsigned long c;
+{
+  voidfunc c0 = command (c);
+  if (c0 == F12 && (keyshift & altshift_mask) == alt_mask) {
+	return toggleKEYMAP;
+  } else if (c0 == F12 && (keyshift & ctrlshift_mask) == ctrl_mask) {
+	return setupKEYMAP;
+  } else {
+	return c0;
+  }
+}
+
 /**
    Read prompt input.
  * Input () reads a string and echoes it on the prompt line.
@@ -785,14 +884,13 @@ input (inbuf, clearfl, term_input)
   char * ptr;
   unsigned long c;
   unsigned long mnemoc;
-  char fkeyshift;
   char * term_chars;
 
   ptr = inbuf;
   * ptr = '\0';
   bottom_lines = 0;
 
-  while (quit == False) {
+  while (! quit) {
      if (lpos >= XBREAK_bottom) {
 	pagewrapped = True;
      }
@@ -835,7 +933,7 @@ input (inbuf, clearfl, term_input)
 
 	/* restore prompt display, remove mnemonic */
 	while ((char *) ptr > mnemo) {
-		ptr = erase1 (inbuf, ptr);
+		ptr = erase1 (inbuf, ptr, True);
 	}
 
 	mnemo = NIL_PTR;
@@ -846,149 +944,76 @@ input (inbuf, clearfl, term_input)
      } else if (c == quit_char) {
 	ring_bell ();
 	quit = True;
-     } else if (allow_keymap && command (c) == toggleKEYMAP) {
+     } else if (allow_keymap && command1 (c) == toggleKEYMAP) {
 	reverse_off ();
 	toggleKEYMAP ();
+	if (flags_changed) {
+		displayflags ();
+	}
 	redraw_prompt ();
-     } else if (allow_keymap && command (c) == setupKEYMAP) {
+     } else if (allow_keymap && command1 (c) == setupKEYMAP) {
 	reverse_off ();
 	setupKEYMAP ();
+	if (flags_changed) {
+		displayflags ();
+	}
 	redraw_prompt ();
-     } else
-     switch (c) {
-
-	case '\b' :		/* Erase previous char */
-	case '\177' /* DEL */ :
-	    if (mnemo == NIL_PTR ? ptr > inbuf : (char *) ptr > mnemo) {
-		ptr = erase1 (inbuf, ptr);
-	    } else {
+     } else if (command (c) == DPC) {
+	if (mnemo == NIL_PTR ? ptr > inbuf : (char *) ptr > mnemo) {
+		ptr = erase1 (inbuf, ptr, ! (keyshift & ctrl_mask));
+	} else {
 		ring_bell ();
-	    }
-	    break;
-
-	case '\033' :
-		quit = True;
-		break;
-
-	case '\n' :		/* End of input */
-	case '\r' :
-		/* If inbuf is empty clear status line */
-		return (ptr == inbuf && clearfl) ?
-			NO_INPUT : FINE;
-
-	case (character) FUNcmd :
-		if (keyproc == insert_diaeresis) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, diaeresis (c));
-		} else if (keyproc == insert_acute) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, acute (c));
-		} else if (keyproc == insert_grave) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, grave (c));
-		} else if (keyproc == insert_circumflex) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, circumflex (c));
-		} else if (keyproc == insert_tilde) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, tilde (c));
-		} else if (keyproc == insert_angstrom) {
-			fkeyshift = keyshift;
-			c = readcharacter ();
-			keyshift = fkeyshift;
-			ptr = input_character (inbuf, ptr, angstrom (c));
-/*
-		} else if (allow_keymap && keyproc == toggleKEYMAP) {
-			reverse_off ();
-			toggleKEYMAP ();
-			redraw_prompt ();
-		} else if (allow_keymap && keyproc == setupKEYMAP) {
-			reverse_off ();
-			setupKEYMAP ();
-			redraw_prompt ();
-*/
+	}
+     } else if (command (c) == CTRLINS) {
+	c = readcharacter_unicode ();
+	if (c == FUNcmd) {
+		struct prefixspec * ps;
+		keyshift |= ctrl_mask;
+		ps = lookup_prefix (keyproc, keyshift);
+		if (ps) {
+			ptr = input_prefix (inbuf, ptr, ps);
 		} else {
 			ring_bell ();
 		}
-		break;
-
-	default :
-		if (c == control_prefix /* ^V/^P/^Q */) {
-			c = readcharacter ();
-			if (
-			    pc_term && ! (utf8_text && utf8_input == False)
-			    ? c == DOS_ring
-			    : c == UNI_ring
-			   )
-			{
-				ptr = input_character (inbuf, ptr, angstrom (readcharacter ()));
-			} else if (
-			    pc_term && ! (utf8_text && utf8_input == False)
-			    ? c == DOS_acute
-			    : c == UNI_acute
-			   )
-			{
-				ptr = input_character (inbuf, ptr, acute (readcharacter ()));
-			} else switch (c) {
-			case '"': {
-				 ptr = input_character (inbuf, ptr, diaeresis (readcharacter ()));
-				 break;
-				}
-			case '\'': {
-				 ptr = input_character (inbuf, ptr, acute (readcharacter ()));
-				 break;
-				}
-			case '`': {
-				 ptr = input_character (inbuf, ptr, grave (readcharacter ()));
-				 break;
-				}
-			case '^': {
-				 ptr = input_character (inbuf, ptr, circumflex (readcharacter ()));
-				 break;
-				}
-			case '~': {
-				 ptr = input_character (inbuf, ptr, tilde (readcharacter ()));
-				 break;
-				}
-			default:
-				if (c == '#') {
-				    unsigned long cx;
-				    do {
-					cx = get_num_char (& c, max_char_value ());
-					if (quit) {
-						quit = False;
-					} else {
-						ptr = input_character (inbuf, ptr, c);
-					}
-				    } while (cx == ' ');
-				} else if (c == ' ') {
-					/* mark mnemonic start position 
-					   in input buffer */
-					mnemo = (char *) ptr;
-					diagdisp_off ();
-				} else if ( (c > ' ' && c <= 'Z')
-				            || (c > '^' && c != 0x7F) ) {
-					c = compose (c, readcharacter ());
-					ptr = input_character (inbuf, ptr, c);
-				} else {
-					ptr = input_character (inbuf, ptr, c);
-				}
-			}
-
+	} else if (c == '#') {
+	    unsigned long cx;
+	    do {
+		cx = get_num_char (& c, max_char_value ());
+		if (quit) {
+			quit = False;
 		} else {
-			/* c != control_prefix */
 			ptr = input_character (inbuf, ptr, c);
 		}
+	    } while (cx == ' ');
+	} else if (c == ' ') {
+		/* mark mnemonic start position 
+		   in input buffer */
+		mnemo = (char *) ptr;
+		diagdisp_off ();
+	} else if (c > ' ' && c != 0x7F) {
+		c = compose (c, readcharacter_unicode ());
+		if (no_char (c)) {
+			ring_bell ();
+		} else {
+			ptr = input_character (inbuf, ptr, c);
+		}
+	} else {
+		ptr = input_character (inbuf, ptr, c);
+	}
+     } else if (c == '\033') {
+	quit = True;
+     } else if (c == '\n' || c == '\r') {
+	/* if inbuf is empty clear status line */
+	return (ptr == inbuf && clearfl) ? NO_INPUT : FINE;
+     } else if (c == FUNcmd) {
+	struct prefixspec * ps = lookup_prefix (keyproc, keyshift);
+	if (ps) {
+		ptr = input_prefix (inbuf, ptr, ps);
+	} else {
+		ring_bell ();
+	}
+     } else {
+	ptr = input_character (inbuf, ptr, c);
      }
   }
   quit = False;
@@ -997,19 +1022,64 @@ input (inbuf, clearfl, term_input)
 
 /*
  * Show concatenation of s1 and s2 on the status line (bottom of screen)
- * If revfl is True, turn on reverse video on both strings. Set stat_visible
- * only if bottom_line is visible.
- * The return value is FINE except for get_string, where it is taken
- * from the call to input ().
-status_line (str1, str2) is (void) bottom_line (ON, (str1), (str2), NIL_PTR, False, "")
+   Set stat_visible only if bottom_line is visible.
+   The return value is FINE except for get_string, where it is taken 
+   from the call to input ().
+status_line (str1, str2) is (void) bottom_line (True, (str1), (str2), NIL_PTR, False, "")
 status_msg (str)	 is status_line (str, NIL_PTR)
-status_beg (str)	 is (void) bottom_line (ON, (str), NIL_PTR, NIL_PTR, True, "")
-error (str1)		 is (void) bottom_line (ON, (str1), NIL_PTR, NIL_PTR, False, "")
-error2 (str1, str2)	 is (void) bottom_line (ON, (str1), (str2), NIL_PTR, False, "")
-clear_status ()		 is (void) bottom_line (OFF, NIL_PTR, NIL_PTR, NIL_PTR, False, "")
+status_beg (str)	 is (void) bottom_line (True, (str), NIL_PTR, NIL_PTR, True, "")
+error (str1)		 is (void) bottom_line (True, (str1), NIL_PTR, NIL_PTR, False, "")
+error2 (str1, str2)	 is (void) bottom_line (True, (str1), (str2), NIL_PTR, False, "")
+clear_status ()		 is (void) bottom_line (False, NIL_PTR, NIL_PTR, NIL_PTR, False, "")
 get_string (str1, str2, fl, term_chars)
-			 is bottom_line (ON, (str1), NIL_PTR, (str2), fl, term_chars)
+			 is bottom_line (True, (str1), NIL_PTR, (str2), fl, term_chars)
  */
+
+void
+status_uni (msg)
+  char * msg;
+{
+  FLAG save_utf8_text = utf8_text;
+  utf8_text = True;
+  /* don't need to save and set utf16_file = False; */
+
+  bottom_line (VALID, msg, NIL_PTR, NIL_PTR, False, "");
+
+  utf8_text = save_utf8_text;
+}
+
+/**
+   get_string_uni prompts a string like get_string_nokeymap
+   but uses Unicode input
+   BEWARE: conflicts with RDwin!
+ */
+int
+get_string_uni (prompt, inbuf, statfl, term_input)
+  char * prompt;
+  char * inbuf;
+  FLAG statfl;
+  char * term_input;
+{
+  int ret;
+
+  /* push text encoding, temp. set to utf8 */
+  FLAG save_utf8_text = utf8_text;
+  FLAG save_cjk_text = cjk_text;
+  FLAG save_mapped_text = mapped_text;
+  utf8_text = True;
+  cjk_text = False;
+  mapped_text = False;
+
+  ret = get_string_nokeymap (prompt, inbuf, statfl, term_input);
+
+  /* pop text encoding */
+  utf8_text = save_utf8_text;
+  cjk_text = save_cjk_text;
+  mapped_text = save_mapped_text;
+
+  return ret;
+}
+
 void
 rd_bottom_line ()
 {
@@ -1017,22 +1087,37 @@ rd_bottom_line ()
   diagdisp_on ();
   if (lastinbuf == NIL_PTR) {
 	lpos = 0;
-	printlim_string (status_buf, XBREAK_bottom);
+	if (lastattrib == VALID) {
+		FLAG save_utf8_text = utf8_text;
+		utf8_text = True;
+		printlim_string (status_buf, XBREAK_bottom, True);
+		utf8_text = save_utf8_text;
+	} else {
+		printlim_string (status_buf, XBREAK_bottom, False);
+	}
   } else {
 	lpos = 0;
 	print_string (status_buf);
 	print_string (lastinbuf);
   }
-  if (input_active == False) {
+  if (! input_active) {
 	diagdisp_off ();
 	set_cursor_xy ();	/* Set cursor back to old position */
   }
   flush ();	/* Perform the actual screen output */
 }
 
+/**
+   Clear or display status line, read prompt if desired.
+   s1, s2: strings to display
+   inbuf: buffer for prompt to read
+   attrib: True: turn on reverse video on both strings
+           False: don't (clear_status)
+           VALID: interpret control chars in string for attributes
+ */
 int
-bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
-  FLAG revfl;
+bottom_line (attrib, s1, s2, inbuf, statfl, term_input)
+  FLAG attrib;
   char * s1;
   char * s2;
   char * inbuf;
@@ -1045,6 +1130,7 @@ bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
 	* inbuf = '\0';
   }
   lastinbuf = inbuf;
+  lastattrib = attrib;
 
   if (pagewrapped) {
 	status_buf [0] = '\0';
@@ -1052,16 +1138,19 @@ bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
 	pagewrapped = False;
   }
 
-  build_string (status_buf, " %s%s ", unnull (s1), unnull (s2));
-		/* (s1 == NIL_PTR) ? "" : s1, (s2 == NIL_PTR) ? "" : s2); */
+  if (attrib == VALID && s1 && * s1 == '') {
+	build_string (status_buf, "%s%s", unnull (s1), unnull (s2));
+  } else {
+	build_string (status_buf, " %s%s ", unnull (s1), unnull (s2));
+  }
 
-  if (revfl == ON && stat_visible) {
+  if (attrib != False && stat_visible) {
 	set_cursor (0, YMAX);
 	clear_lastline ();
   }
 
   set_cursor (0, YMAX);
-  if (revfl == ON) {		/* Print rev. start sequence */
+  if (attrib != False) {	/* Print rev. start sequence */
 	diagdisp_on ();
 	stat_visible = True;
   } else {			/* Used as clear_status () */
@@ -1070,7 +1159,7 @@ bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
   }
   if (inbuf == NIL_PTR) {
 	lpos = 0;
-	printlim_string (status_buf, XBREAK_bottom);
+	printlim_string (status_buf, XBREAK_bottom, attrib == VALID);
   } else {
 	lpos = 0;
 	print_string (status_buf);
@@ -1091,7 +1180,7 @@ bottom_line (revfl, s1, s2, inbuf, statfl, term_input)
 		diagdisp_on ();
 		if (lastinbuf == NIL_PTR) {
 			lpos = 0;
-			printlim_string (status_buf, XBREAK_bottom);
+			printlim_string (status_buf, XBREAK_bottom, attrib == VALID);
 		} else {
 			lpos = 0;
 			print_string (status_buf);
@@ -1131,7 +1220,7 @@ get_filename (message, file)
   char * filei;
   char file1 [maxLINE_LEN];
 
-  if (file [0] == '~' && file [1] == '/') {
+  if (file [0] == '~' && (file [1] == '/' || file [1] == '\0')) {
 	filei = file; filei ++;
 	build_string (file1, "%s%s", envvar ("HOME"), filei);
 	build_string (file, file1);
@@ -1175,14 +1264,14 @@ get_number (message, firstdigit, result)
   if (ch == quit_char || ch == '\033') {
 	quit = True;
   }
-  if (quit == False && (ch < '0' || ch > '9')) {
+  if (! quit && (ch < '0' || ch > '9')) {
 	error ("Bad number");
 	return ERRORS;
   }
 
 /* Convert input to a decimal number */
   value = 0;
-  while (quit == False && ((ch >= '0' && ch <= '9')
+  while (! quit && ((ch >= '0' && ch <= '9')
 			   || ch == '\b' || ch == '\177')) {
 	if (ch == '\b' || ch == '\177') {
 		if (i > 0) {
@@ -1234,10 +1323,14 @@ prompt_num_char (result, maxvalue)
   int base = 16;
   unsigned long maxfill = 0;
 
-  status_beg ("Enter character code value (hex / # octal / = decimal) ...");
+  if (cjk_text || mapped_text) {
+	status_beg ("Enter character code value (hex / # oct / = dec / u unicode) ...");
+  } else {
+	status_beg ("Enter character code value (hex / # oct / = dec) ...");
+  }
 
   i = 0;
-  while (maxfill < maxvalue && ok == False) {
+  while (maxfill < maxvalue && ! ok) {
 	ch = readcharacter ();
 	if (i == 0) {
 		if (ch == '#') {
@@ -1319,7 +1412,7 @@ prompt_num_char (result, maxvalue)
   }
 
   if (uniinput && (cjk_text || mapped_text)) {
-	* result = cjk (value);
+	* result = encodedchar (value);
   } else {
 	* result = value;
   }
