@@ -44,8 +44,7 @@
 	{0x00D7, 0x00D7, "Common", "Symbol"},
 	{0x00D8, 0x00F6, "Latin", "Letter"},
 	{0x00F7, 0x00F7, "Common", "Symbol"},
-	{0x00F8, 0x0236, "Latin", "Letter"},
-	{0x0250, 0x02B8, "Latin", "Letter"},
+	{0x00F8, 0x02B8, "Latin", "Letter"},
 	{0x02B9, 0x02C1, "Common", "Letter"},
 	{0x02C2, 0x02C5, "Common", "Symbol"},
 	{0x02C6, 0x02D1, "Common", "Letter"},
@@ -54,10 +53,9 @@
 	{0x02E5, 0x02ED, "Common", "Symbol"},
 	{0x02EE, 0x02EE, "Common", "Letter"},
 	{0x02EF, 0x02FF, "Common", "Symbol"},
-	{0x0300, 0x0357, "Inherited", "Mark"},
-	{0x035D, 0x036F, "Inherited", "Mark"},
+	{0x0300, 0x036F, "Inherited", "Mark"},
 	{0x0374, 0x0375, "Greek", "Symbol"},
-	{0x037A, 0x037A, "Greek", "Letter"},
+	{0x037A, 0x037D, "Greek", "Letter"},
 	{0x037E, 0x037E, "Common", "Punctuation"},
 	{0x0384, 0x0385, "Greek", "Symbol"},
 	{0x0386, 0x0386, "Greek", "Letter"},
@@ -66,47 +64,48 @@
 	{0x038C, 0x038C, "Greek", "Letter"},
 	{0x038E, 0x03A1, "Greek", "Letter"},
 	{0x03A3, 0x03CE, "Greek", "Letter"},
-	{0x03D0, 0x03F5, "Greek", "Letter"},
+	{0x03D0, 0x03E1, "Greek", "Letter"},
+	{0x03E2, 0x03EF, "Coptic", "Letter"},
+	{0x03F0, 0x03F5, "Greek", "Letter"},
 	{0x03F6, 0x03F6, "Greek", "Symbol"},
-	{0x03F7, 0x03FB, "Greek", "Letter"},
+	{0x03F7, 0x03FF, "Greek", "Letter"},
 	{0x0400, 0x0481, "Cyrillic", "Letter"},
 	{0x0482, 0x0482, "Cyrillic", "Symbol"},
 	{0x0483, 0x0486, "Cyrillic", "Mark"},
 	{0x0488, 0x0489, "Cyrillic", "Mark"},
-	{0x048A, 0x04CE, "Cyrillic", "Letter"},
-	{0x04D0, 0x04F5, "Cyrillic", "Letter"},
-	{0x04F8, 0x04F9, "Cyrillic", "Letter"},
-	{0x0500, 0x050F, "Cyrillic", "Letter"},
+	{0x048A, 0x0513, "Cyrillic", "Letter"},
 	{0x0531, 0x0556, "Armenian", "Letter"},
 	{0x0559, 0x0559, "Armenian", "Letter"},
 	{0x055A, 0x055F, "Armenian", "Punctuation"},
 	{0x0561, 0x0587, "Armenian", "Letter"},
 	{0x0589, 0x0589, "Common", "Punctuation"},
 	{0x058A, 0x058A, "Armenian", "Punctuation"},
-	{0x0591, 0x05A1, "Hebrew", "Mark"},
-	{0x05A3, 0x05B9, "Hebrew", "Mark"},
-	{0x05BB, 0x05BD, "Hebrew", "Mark"},
+	{0x0591, 0x05BD, "Hebrew", "Mark"},
 	{0x05BE, 0x05BE, "Hebrew", "Punctuation"},
 	{0x05BF, 0x05BF, "Hebrew", "Mark"},
 	{0x05C0, 0x05C0, "Hebrew", "Punctuation"},
 	{0x05C1, 0x05C2, "Hebrew", "Mark"},
 	{0x05C3, 0x05C3, "Hebrew", "Punctuation"},
-	{0x05C4, 0x05C4, "Hebrew", "Mark"},
+	{0x05C4, 0x05C5, "Hebrew", "Mark"},
+	{0x05C6, 0x05C6, "Hebrew", "Punctuation"},
+	{0x05C7, 0x05C7, "Hebrew", "Mark"},
 	{0x05D0, 0x05EA, "Hebrew", "Letter"},
 	{0x05F0, 0x05F2, "Hebrew", "Letter"},
 	{0x05F3, 0x05F4, "Hebrew", "Punctuation"},
 	{0x0600, 0x0603, "Common", "Format"},
+	{0x060B, 0x060B, "Arabic", "Currency Symbol"},
 	{0x060C, 0x060C, "Common", "Punctuation"},
 	{0x060D, 0x060D, "Arabic", "Punctuation"},
 	{0x060E, 0x060F, "Arabic", "Symbol"},
 	{0x0610, 0x0615, "Arabic", "Mark"},
 	{0x061B, 0x061B, "Common", "Punctuation"},
+	{0x061E, 0x061E, "Arabic", "Punctuation"},
 	{0x061F, 0x061F, "Common", "Punctuation"},
 	{0x0621, 0x063A, "Arabic", "Letter"},
 	{0x0640, 0x0640, "Common", "Letter"},
 	{0x0641, 0x064A, "Arabic", "Letter"},
 	{0x064B, 0x0655, "Inherited", "Mark"},
-	{0x0656, 0x0658, "Arabic", "Mark"},
+	{0x0656, 0x065E, "Arabic", "Mark"},
 	{0x0660, 0x0669, "Common", "Number"},
 	{0x066A, 0x066D, "Arabic", "Punctuation"},
 	{0x066E, 0x066F, "Arabic", "Letter"},
@@ -133,9 +132,17 @@
 	{0x0712, 0x072F, "Syriac", "Letter"},
 	{0x0730, 0x074A, "Syriac", "Mark"},
 	{0x074D, 0x074F, "Syriac", "Letter"},
+	{0x0750, 0x076D, "Arabic", "Letter"},
 	{0x0780, 0x07A5, "Thaana", "Letter"},
 	{0x07A6, 0x07B0, "Thaana", "Mark"},
 	{0x07B1, 0x07B1, "Thaana", "Letter"},
+	{0x07C0, 0x07C9, "Nko", "Number"},
+	{0x07CA, 0x07EA, "Nko", "Letter"},
+	{0x07EB, 0x07F3, "Nko", "Mark"},
+	{0x07F4, 0x07F5, "Nko", "Letter"},
+	{0x07F6, 0x07F6, "Nko", "Symbol"},
+	{0x07F7, 0x07F9, "Nko", "Punctuation"},
+	{0x07FA, 0x07FA, "Nko", "Letter"},
 	{0x0901, 0x0903, "Devanagari", "Mark"},
 	{0x0904, 0x0939, "Devanagari", "Letter"},
 	{0x093C, 0x093C, "Devanagari", "Mark"},
@@ -148,6 +155,7 @@
 	{0x0964, 0x0965, "Common", "Punctuation"},
 	{0x0966, 0x096F, "Devanagari", "Number"},
 	{0x0970, 0x0970, "Common", "Punctuation"},
+	{0x097B, 0x097F, "Devanagari", "Letter"},
 	{0x0981, 0x0983, "Bengali", "Mark"},
 	{0x0985, 0x098C, "Bengali", "Letter"},
 	{0x098F, 0x0990, "Bengali", "Letter"},
@@ -160,6 +168,7 @@
 	{0x09BE, 0x09C4, "Bengali", "Mark"},
 	{0x09C7, 0x09C8, "Bengali", "Mark"},
 	{0x09CB, 0x09CD, "Bengali", "Mark"},
+	{0x09CE, 0x09CE, "Bengali", "Letter"},
 	{0x09D7, 0x09D7, "Bengali", "Mark"},
 	{0x09DC, 0x09DD, "Bengali", "Letter"},
 	{0x09DF, 0x09E1, "Bengali", "Letter"},
@@ -231,13 +240,12 @@
 	{0x0B9E, 0x0B9F, "Tamil", "Letter"},
 	{0x0BA3, 0x0BA4, "Tamil", "Letter"},
 	{0x0BA8, 0x0BAA, "Tamil", "Letter"},
-	{0x0BAE, 0x0BB5, "Tamil", "Letter"},
-	{0x0BB7, 0x0BB9, "Tamil", "Letter"},
+	{0x0BAE, 0x0BB9, "Tamil", "Letter"},
 	{0x0BBE, 0x0BC2, "Tamil", "Mark"},
 	{0x0BC6, 0x0BC8, "Tamil", "Mark"},
 	{0x0BCA, 0x0BCD, "Tamil", "Mark"},
 	{0x0BD7, 0x0BD7, "Tamil", "Mark"},
-	{0x0BE7, 0x0BF2, "Tamil", "Number"},
+	{0x0BE6, 0x0BF2, "Tamil", "Number"},
 	{0x0BF3, 0x0BF8, "Tamil", "Symbol"},
 	{0x0BF9, 0x0BF9, "Tamil", "Currency Symbol"},
 	{0x0BFA, 0x0BFA, "Tamil", "Symbol"},
@@ -267,7 +275,9 @@
 	{0x0CD5, 0x0CD6, "Kannada", "Mark"},
 	{0x0CDE, 0x0CDE, "Kannada", "Letter"},
 	{0x0CE0, 0x0CE1, "Kannada", "Letter"},
+	{0x0CE2, 0x0CE3, "Kannada", "Mark"},
 	{0x0CE6, 0x0CEF, "Kannada", "Number"},
+	{0x0CF1, 0x0CF2, "Kannada", "Symbol"},
 	{0x0D02, 0x0D03, "Malayalam", "Mark"},
 	{0x0D05, 0x0D0C, "Malayalam", "Letter"},
 	{0x0D0E, 0x0D10, "Malayalam", "Letter"},
@@ -350,6 +360,7 @@
 	{0x0FC6, 0x0FC6, "Tibetan", "Mark"},
 	{0x0FC7, 0x0FCC, "Tibetan", "Symbol"},
 	{0x0FCF, 0x0FCF, "Tibetan", "Symbol"},
+	{0x0FD0, 0x0FD1, "Tibetan", "Punctuation"},
 	{0x1000, 0x1021, "Myanmar", "Letter"},
 	{0x1023, 0x1027, "Myanmar", "Letter"},
 	{0x1029, 0x102A, "Myanmar", "Letter"},
@@ -360,38 +371,34 @@
 	{0x1050, 0x1055, "Myanmar", "Letter"},
 	{0x1056, 0x1059, "Myanmar", "Mark"},
 	{0x10A0, 0x10C5, "Georgian", "Letter"},
-	{0x10D0, 0x10F8, "Georgian", "Letter"},
+	{0x10D0, 0x10FA, "Georgian", "Letter"},
 	{0x10FB, 0x10FB, "Common", "Punctuation"},
+	{0x10FC, 0x10FC, "Georgian", "Letter"},
 	{0x1100, 0x1159, "Hangul", "Letter"},
 	{0x115F, 0x11A2, "Hangul", "Letter"},
 	{0x11A8, 0x11F9, "Hangul", "Letter"},
-	{0x1200, 0x1206, "Ethiopic", "Letter"},
-	{0x1208, 0x1246, "Ethiopic", "Letter"},
-	{0x1248, 0x1248, "Ethiopic", "Letter"},
+	{0x1200, 0x1248, "Ethiopic", "Letter"},
 	{0x124A, 0x124D, "Ethiopic", "Letter"},
 	{0x1250, 0x1256, "Ethiopic", "Letter"},
 	{0x1258, 0x1258, "Ethiopic", "Letter"},
 	{0x125A, 0x125D, "Ethiopic", "Letter"},
-	{0x1260, 0x1286, "Ethiopic", "Letter"},
-	{0x1288, 0x1288, "Ethiopic", "Letter"},
+	{0x1260, 0x1288, "Ethiopic", "Letter"},
 	{0x128A, 0x128D, "Ethiopic", "Letter"},
-	{0x1290, 0x12AE, "Ethiopic", "Letter"},
-	{0x12B0, 0x12B0, "Ethiopic", "Letter"},
+	{0x1290, 0x12B0, "Ethiopic", "Letter"},
 	{0x12B2, 0x12B5, "Ethiopic", "Letter"},
 	{0x12B8, 0x12BE, "Ethiopic", "Letter"},
 	{0x12C0, 0x12C0, "Ethiopic", "Letter"},
 	{0x12C2, 0x12C5, "Ethiopic", "Letter"},
-	{0x12C8, 0x12CE, "Ethiopic", "Letter"},
-	{0x12D0, 0x12D6, "Ethiopic", "Letter"},
-	{0x12D8, 0x12EE, "Ethiopic", "Letter"},
-	{0x12F0, 0x130E, "Ethiopic", "Letter"},
-	{0x1310, 0x1310, "Ethiopic", "Letter"},
+	{0x12C8, 0x12D6, "Ethiopic", "Letter"},
+	{0x12D8, 0x1310, "Ethiopic", "Letter"},
 	{0x1312, 0x1315, "Ethiopic", "Letter"},
-	{0x1318, 0x131E, "Ethiopic", "Letter"},
-	{0x1320, 0x1346, "Ethiopic", "Letter"},
-	{0x1348, 0x135A, "Ethiopic", "Letter"},
+	{0x1318, 0x135A, "Ethiopic", "Letter"},
+	{0x135F, 0x135F, "Ethiopic", "Mark"},
+	{0x1360, 0x1360, "Ethiopic", "Symbol"},
 	{0x1361, 0x1368, "Ethiopic", "Punctuation"},
 	{0x1369, 0x137C, "Ethiopic", "Number"},
+	{0x1380, 0x138F, "Ethiopic", "Letter"},
+	{0x1390, 0x1399, "Ethiopic", "Symbol"},
 	{0x13A0, 0x13F4, "Cherokee", "Letter"},
 	{0x1401, 0x166C, "Canadian Aboriginal", "Letter"},
 	{0x166D, 0x166E, "Canadian Aboriginal", "Punctuation"},
@@ -424,7 +431,11 @@
 	{0x17DD, 0x17DD, "Khmer", "Mark"},
 	{0x17E0, 0x17E9, "Khmer", "Number"},
 	{0x17F0, 0x17F9, "Khmer", "Number"},
-	{0x1800, 0x180A, "Mongolian", "Punctuation"},
+	{0x1800, 0x1801, "Mongolian", "Punctuation"},
+	{0x1802, 0x1803, "Common", "Punctuation"},
+	{0x1804, 0x1804, "Mongolian", "Punctuation"},
+	{0x1805, 0x1805, "Common", "Punctuation"},
+	{0x1806, 0x180A, "Mongolian", "Punctuation"},
 	{0x180B, 0x180D, "Mongolian", "Mark"},
 	{0x180E, 0x180E, "Mongolian", "Separator"},
 	{0x1810, 0x1819, "Mongolian", "Number"},
@@ -439,7 +450,25 @@
 	{0x1946, 0x194F, "Limbu", "Number"},
 	{0x1950, 0x196D, "Tai Le", "Letter"},
 	{0x1970, 0x1974, "Tai Le", "Letter"},
+	{0x1980, 0x19A9, "New Tai Lue", "Letter"},
+	{0x19B0, 0x19C0, "New Tai Lue", "Mark"},
+	{0x19C1, 0x19C7, "New Tai Lue", "Letter"},
+	{0x19C8, 0x19C9, "New Tai Lue", "Mark"},
+	{0x19D0, 0x19D9, "New Tai Lue", "Number"},
+	{0x19DE, 0x19DF, "New Tai Lue", "Punctuation"},
 	{0x19E0, 0x19FF, "Khmer", "Symbol"},
+	{0x1A00, 0x1A16, "Buginese", "Letter"},
+	{0x1A17, 0x1A1B, "Buginese", "Mark"},
+	{0x1A1E, 0x1A1F, "Buginese", "Punctuation"},
+	{0x1B00, 0x1B04, "Balinese", "Mark"},
+	{0x1B05, 0x1B33, "Balinese", "Letter"},
+	{0x1B34, 0x1B44, "Balinese", "Mark"},
+	{0x1B45, 0x1B4B, "Balinese", "Letter"},
+	{0x1B50, 0x1B59, "Balinese", "Number"},
+	{0x1B5A, 0x1B60, "Balinese", "Punctuation"},
+	{0x1B61, 0x1B6A, "Balinese", "Symbol"},
+	{0x1B6B, 0x1B73, "Balinese", "Mark"},
+	{0x1B74, 0x1B7C, "Balinese", "Symbol"},
 	{0x1D00, 0x1D25, "Latin", "Letter"},
 	{0x1D26, 0x1D2A, "Greek", "Letter"},
 	{0x1D2B, 0x1D2B, "Cyrillic", "Letter"},
@@ -447,7 +476,12 @@
 	{0x1D5D, 0x1D61, "Greek", "Letter"},
 	{0x1D62, 0x1D65, "Latin", "Letter"},
 	{0x1D66, 0x1D6A, "Greek", "Letter"},
-	{0x1D6B, 0x1D6B, "Latin", "Letter"},
+	{0x1D6B, 0x1D77, "Latin", "Letter"},
+	{0x1D78, 0x1D78, "Cyrillic", "Letter"},
+	{0x1D79, 0x1DBE, "Latin", "Letter"},
+	{0x1DBF, 0x1DBF, "Greek", "Letter"},
+	{0x1DC0, 0x1DCA, "Inherited", "Mark"},
+	{0x1DFE, 0x1DFF, "Inherited", "Mark"},
 	{0x1E00, 0x1E9B, "Latin", "Letter"},
 	{0x1EA0, 0x1EF9, "Latin", "Letter"},
 	{0x1F00, 0x1F15, "Greek", "Letter"},
@@ -487,8 +521,7 @@
 	{0x2044, 0x2044, "Common", "Symbol"},
 	{0x2045, 0x2051, "Common", "Punctuation"},
 	{0x2052, 0x2052, "Common", "Symbol"},
-	{0x2053, 0x2054, "Common", "Punctuation"},
-	{0x2057, 0x2057, "Common", "Punctuation"},
+	{0x2053, 0x205E, "Common", "Punctuation"},
 	{0x205F, 0x205F, "Common", "Separator"},
 	{0x2060, 0x2063, "Common", "Format"},
 	{0x206A, 0x206F, "Common", "Format"},
@@ -501,8 +534,9 @@
 	{0x2080, 0x2089, "Common", "Number"},
 	{0x208A, 0x208C, "Common", "Symbol"},
 	{0x208D, 0x208E, "Common", "Punctuation"},
-	{0x20A0, 0x20B1, "Common", "Currency Symbol"},
-	{0x20D0, 0x20EA, "Inherited", "Mark"},
+	{0x2090, 0x2094, "Latin", "Letter"},
+	{0x20A0, 0x20B5, "Common", "Currency Symbol"},
+	{0x20D0, 0x20EF, "Inherited", "Mark"},
 	{0x2100, 0x2101, "Common", "Symbol"},
 	{0x2102, 0x2102, "Common", "Letter"},
 	{0x2103, 0x2106, "Common", "Symbol"},
@@ -524,28 +558,27 @@
 	{0x212C, 0x212D, "Common", "Letter"},
 	{0x212E, 0x212E, "Common", "Symbol"},
 	{0x212F, 0x2131, "Common", "Letter"},
-	{0x2132, 0x2132, "Common", "Symbol"},
+	{0x2132, 0x2132, "Latin", "Letter"},
 	{0x2133, 0x2139, "Common", "Letter"},
 	{0x213A, 0x213B, "Common", "Symbol"},
-	{0x213D, 0x213F, "Common", "Letter"},
+	{0x213C, 0x213F, "Common", "Letter"},
 	{0x2140, 0x2144, "Common", "Symbol"},
 	{0x2145, 0x2149, "Common", "Letter"},
-	{0x214A, 0x214B, "Common", "Symbol"},
-	{0x2153, 0x2183, "Common", "Number"},
+	{0x214A, 0x214D, "Common", "Symbol"},
+	{0x214E, 0x214E, "Latin", "Letter"},
+	{0x2153, 0x2182, "Common", "Number"},
+	{0x2183, 0x2183, "Common", "Letter"},
+	{0x2184, 0x2184, "Latin", "Letter"},
 	{0x2190, 0x2328, "Common", "Symbol"},
 	{0x2329, 0x232A, "Common", "Punctuation"},
-	{0x232B, 0x23B3, "Common", "Symbol"},
-	{0x23B4, 0x23B6, "Common", "Punctuation"},
-	{0x23B7, 0x23D0, "Common", "Symbol"},
+	{0x232B, 0x23E7, "Common", "Symbol"},
 	{0x2400, 0x2426, "Common", "Symbol"},
 	{0x2440, 0x244A, "Common", "Symbol"},
 	{0x2460, 0x249B, "Common", "Number"},
 	{0x249C, 0x24E9, "Common", "Symbol"},
 	{0x24EA, 0x24FF, "Common", "Number"},
-	{0x2500, 0x2617, "Common", "Symbol"},
-	{0x2619, 0x267D, "Common", "Symbol"},
-	{0x2680, 0x2691, "Common", "Symbol"},
-	{0x26A0, 0x26A1, "Common", "Symbol"},
+	{0x2500, 0x269C, "Common", "Symbol"},
+	{0x26A0, 0x26B2, "Common", "Symbol"},
 	{0x2701, 0x2704, "Common", "Symbol"},
 	{0x2706, 0x2709, "Common", "Symbol"},
 	{0x270C, 0x2727, "Common", "Symbol"},
@@ -560,6 +593,9 @@
 	{0x2794, 0x2794, "Common", "Symbol"},
 	{0x2798, 0x27AF, "Common", "Symbol"},
 	{0x27B1, 0x27BE, "Common", "Symbol"},
+	{0x27C0, 0x27C4, "Common", "Symbol"},
+	{0x27C5, 0x27C6, "Common", "Punctuation"},
+	{0x27C7, 0x27CA, "Common", "Symbol"},
 	{0x27D0, 0x27E5, "Common", "Symbol"},
 	{0x27E6, 0x27EB, "Common", "Punctuation"},
 	{0x27F0, 0x27FF, "Common", "Symbol"},
@@ -570,7 +606,31 @@
 	{0x29D8, 0x29DB, "Common", "Punctuation"},
 	{0x29DC, 0x29FB, "Common", "Symbol"},
 	{0x29FC, 0x29FD, "Common", "Punctuation"},
-	{0x29FE, 0x2B0D, "Common", "Symbol"},
+	{0x29FE, 0x2B1A, "Common", "Symbol"},
+	{0x2B20, 0x2B23, "Common", "Symbol"},
+	{0x2C00, 0x2C2E, "Glagolitic", "Letter"},
+	{0x2C30, 0x2C5E, "Glagolitic", "Letter"},
+	{0x2C60, 0x2C6C, "Latin", "Letter"},
+	{0x2C74, 0x2C77, "Latin", "Letter"},
+	{0x2C80, 0x2CE4, "Coptic", "Letter"},
+	{0x2CE5, 0x2CEA, "Coptic", "Symbol"},
+	{0x2CF9, 0x2CFC, "Coptic", "Punctuation"},
+	{0x2CFD, 0x2CFD, "Coptic", "Number"},
+	{0x2CFE, 0x2CFF, "Coptic", "Punctuation"},
+	{0x2D00, 0x2D25, "Georgian", "Letter"},
+	{0x2D30, 0x2D65, "Tifinagh", "Letter"},
+	{0x2D6F, 0x2D6F, "Tifinagh", "Letter"},
+	{0x2D80, 0x2D96, "Ethiopic", "Letter"},
+	{0x2DA0, 0x2DA6, "Ethiopic", "Letter"},
+	{0x2DA8, 0x2DAE, "Ethiopic", "Letter"},
+	{0x2DB0, 0x2DB6, "Ethiopic", "Letter"},
+	{0x2DB8, 0x2DBE, "Ethiopic", "Letter"},
+	{0x2DC0, 0x2DC6, "Ethiopic", "Letter"},
+	{0x2DC8, 0x2DCE, "Ethiopic", "Letter"},
+	{0x2DD0, 0x2DD6, "Ethiopic", "Letter"},
+	{0x2DD8, 0x2DDE, "Ethiopic", "Letter"},
+	{0x2E00, 0x2E17, "Common", "Punctuation"},
+	{0x2E1C, 0x2E1D, "Common", "Punctuation"},
 	{0x2E80, 0x2E99, "Han", "Symbol"},
 	{0x2E9B, 0x2EF3, "Han", "Symbol"},
 	{0x2F00, 0x2FD5, "Han", "Symbol"},
@@ -588,7 +648,7 @@
 	{0x3021, 0x3029, "Han", "Number"},
 	{0x302A, 0x302F, "Inherited", "Mark"},
 	{0x3030, 0x3030, "Common", "Punctuation"},
-	{0x3031, 0x3035, "Katakana Or Hiragana", "Letter"},
+	{0x3031, 0x3035, "Common", "Letter"},
 	{0x3036, 0x3037, "Common", "Symbol"},
 	{0x3038, 0x303A, "Han", "Number"},
 	{0x303B, 0x303B, "Han", "Letter"},
@@ -597,7 +657,7 @@
 	{0x303E, 0x303F, "Common", "Symbol"},
 	{0x3041, 0x3096, "Hiragana", "Letter"},
 	{0x3099, 0x309A, "Inherited", "Mark"},
-	{0x309B, 0x309C, "Katakana Or Hiragana", "Symbol"},
+	{0x309B, 0x309C, "Common", "Symbol"},
 	{0x309D, 0x309F, "Hiragana", "Letter"},
 	{0x30A0, 0x30A0, "Common", "Punctuation"},
 	{0x30A1, 0x30FA, "Katakana", "Letter"},
@@ -610,6 +670,7 @@
 	{0x3192, 0x3195, "Common", "Number"},
 	{0x3196, 0x319F, "Common", "Symbol"},
 	{0x31A0, 0x31B7, "Bopomofo", "Letter"},
+	{0x31C0, 0x31CF, "Common", "Symbol"},
 	{0x31F0, 0x31FF, "Katakana", "Letter"},
 	{0x3200, 0x321E, "Hangul", "Symbol"},
 	{0x3220, 0x3229, "Common", "Number"},
@@ -617,7 +678,7 @@
 	{0x3250, 0x3250, "Common", "Symbol"},
 	{0x3251, 0x325F, "Common", "Number"},
 	{0x3260, 0x327D, "Hangul", "Symbol"},
-	{0x327F, 0x327F, "Common", "Symbol"},
+	{0x327E, 0x327F, "Common", "Symbol"},
 	{0x3280, 0x3289, "Common", "Number"},
 	{0x328A, 0x32B0, "Common", "Symbol"},
 	{0x32B1, 0x32BF, "Common", "Number"},
@@ -625,13 +686,27 @@
 	{0x3300, 0x33FF, "Common", "Symbol"},
 	{0x3400, 0x4DB5, "Han", "Letter"},
 	{0x4DC0, 0x4DFF, "Common", "Symbol"},
-	{0x4E00, 0x9FA5, "Han", "Letter"},
+	{0x4E00, 0x9FBB, "Han", "Letter"},
 	{0xA000, 0xA48C, "Yi", "Letter"},
 	{0xA490, 0xA4C6, "Yi", "Symbol"},
+	{0xA700, 0xA716, "Common", "Symbol"},
+	{0xA717, 0xA71A, "Common", "Letter"},
+	{0xA720, 0xA721, "Common", "Symbol"},
+	{0xA800, 0xA801, "Syloti Nagri", "Letter"},
+	{0xA802, 0xA802, "Syloti Nagri", "Mark"},
+	{0xA803, 0xA805, "Syloti Nagri", "Letter"},
+	{0xA806, 0xA806, "Syloti Nagri", "Mark"},
+	{0xA807, 0xA80A, "Syloti Nagri", "Letter"},
+	{0xA80B, 0xA80B, "Syloti Nagri", "Mark"},
+	{0xA80C, 0xA822, "Syloti Nagri", "Letter"},
+	{0xA823, 0xA827, "Syloti Nagri", "Mark"},
+	{0xA828, 0xA82B, "Syloti Nagri", "Symbol"},
+	{0xA840, 0xA873, "Phags Pa", "Letter"},
+	{0xA874, 0xA877, "Phags Pa", "Punctuation"},
 	{0xAC00, 0xD7A3, "Hangul", "Letter"},
-	{0xE000, 0xF8FF, "Common", "Private Use"},
 	{0xF900, 0xFA2D, "Han", "Letter"},
 	{0xFA30, 0xFA6A, "Han", "Letter"},
+	{0xFA70, 0xFAD9, "Han", "Letter"},
 	{0xFB00, 0xFB06, "Latin", "Letter"},
 	{0xFB13, 0xFB17, "Armenian", "Letter"},
 	{0xFB1D, 0xFB1D, "Hebrew", "Letter"},
@@ -653,6 +728,7 @@
 	{0xFDFC, 0xFDFC, "Arabic", "Currency Symbol"},
 	{0xFDFD, 0xFDFD, "Common", "Symbol"},
 	{0xFE00, 0xFE0F, "Inherited", "Mark"},
+	{0xFE10, 0xFE19, "Common", "Punctuation"},
 	{0xFE20, 0xFE23, "Inherited", "Mark"},
 	{0xFE30, 0xFE52, "Common", "Punctuation"},
 	{0xFE54, 0xFE61, "Common", "Punctuation"},
@@ -686,9 +762,9 @@
 	{0xFF5E, 0xFF5E, "Common", "Symbol"},
 	{0xFF5F, 0xFF65, "Common", "Punctuation"},
 	{0xFF66, 0xFF6F, "Katakana", "Letter"},
-	{0xFF70, 0xFF70, "Katakana Or Hiragana", "Letter"},
+	{0xFF70, 0xFF70, "Common", "Letter"},
 	{0xFF71, 0xFF9D, "Katakana", "Letter"},
-	{0xFF9E, 0xFF9F, "Katakana Or Hiragana", "Letter"},
+	{0xFF9E, 0xFF9F, "Common", "Letter"},
 	{0xFFA0, 0xFFBE, "Hangul", "Letter"},
 	{0xFFC2, 0xFFC7, "Hangul", "Letter"},
 	{0xFFCA, 0xFFCF, "Hangul", "Letter"},
@@ -711,12 +787,21 @@
 	{0x10102, 0x10102, "Common", "Symbol"},
 	{0x10107, 0x10133, "Common", "Number"},
 	{0x10137, 0x1013F, "Common", "Symbol"},
+	{0x10140, 0x10178, "Greek", "Number"},
+	{0x10179, 0x10189, "Greek", "Symbol"},
+	{0x1018A, 0x1018A, "Greek", "Number"},
 	{0x10300, 0x1031E, "Old Italic", "Letter"},
 	{0x10320, 0x10323, "Old Italic", "Number"},
-	{0x10330, 0x10349, "Gothic", "Letter"},
+	{0x10330, 0x10340, "Gothic", "Letter"},
+	{0x10341, 0x10341, "Gothic", "Number"},
+	{0x10342, 0x10349, "Gothic", "Letter"},
 	{0x1034A, 0x1034A, "Gothic", "Number"},
 	{0x10380, 0x1039D, "Ugaritic", "Letter"},
 	{0x1039F, 0x1039F, "Ugaritic", "Punctuation"},
+	{0x103A0, 0x103C3, "Old Persian", "Letter"},
+	{0x103C8, 0x103CF, "Old Persian", "Letter"},
+	{0x103D0, 0x103D0, "Old Persian", "Punctuation"},
+	{0x103D1, 0x103D5, "Old Persian", "Number"},
 	{0x10400, 0x1044F, "Deseret", "Letter"},
 	{0x10450, 0x1047F, "Shavian", "Letter"},
 	{0x10480, 0x1049D, "Osmanya", "Letter"},
@@ -727,6 +812,23 @@
 	{0x10837, 0x10838, "Cypriot", "Letter"},
 	{0x1083C, 0x1083C, "Cypriot", "Letter"},
 	{0x1083F, 0x1083F, "Cypriot", "Letter"},
+	{0x10900, 0x10915, "Phoenician", "Letter"},
+	{0x10916, 0x10919, "Phoenician", "Number"},
+	{0x1091F, 0x1091F, "Phoenician", "Punctuation"},
+	{0x10A00, 0x10A00, "Kharoshthi", "Letter"},
+	{0x10A01, 0x10A03, "Kharoshthi", "Mark"},
+	{0x10A05, 0x10A06, "Kharoshthi", "Mark"},
+	{0x10A0C, 0x10A0F, "Kharoshthi", "Mark"},
+	{0x10A10, 0x10A13, "Kharoshthi", "Letter"},
+	{0x10A15, 0x10A17, "Kharoshthi", "Letter"},
+	{0x10A19, 0x10A33, "Kharoshthi", "Letter"},
+	{0x10A38, 0x10A3A, "Kharoshthi", "Mark"},
+	{0x10A3F, 0x10A3F, "Kharoshthi", "Mark"},
+	{0x10A40, 0x10A47, "Kharoshthi", "Number"},
+	{0x10A50, 0x10A58, "Kharoshthi", "Punctuation"},
+	{0x12000, 0x1236E, "Cuneiform", "Letter"},
+	{0x12400, 0x12462, "Cuneiform", "Number"},
+	{0x12470, 0x12473, "Cuneiform", "Punctuation"},
 	{0x1D000, 0x1D0F5, "Common", "Symbol"},
 	{0x1D100, 0x1D126, "Common", "Symbol"},
 	{0x1D12A, 0x1D164, "Common", "Symbol"},
@@ -741,7 +843,11 @@
 	{0x1D18C, 0x1D1A9, "Common", "Symbol"},
 	{0x1D1AA, 0x1D1AD, "Inherited", "Mark"},
 	{0x1D1AE, 0x1D1DD, "Common", "Symbol"},
+	{0x1D200, 0x1D241, "Greek", "Symbol"},
+	{0x1D242, 0x1D244, "Greek", "Mark"},
+	{0x1D245, 0x1D245, "Greek", "Symbol"},
 	{0x1D300, 0x1D356, "Common", "Symbol"},
+	{0x1D360, 0x1D371, "Common", "Number"},
 	{0x1D400, 0x1D454, "Common", "Letter"},
 	{0x1D456, 0x1D49C, "Common", "Letter"},
 	{0x1D49E, 0x1D49F, "Common", "Letter"},
@@ -760,7 +866,7 @@
 	{0x1D540, 0x1D544, "Common", "Letter"},
 	{0x1D546, 0x1D546, "Common", "Letter"},
 	{0x1D54A, 0x1D550, "Common", "Letter"},
-	{0x1D552, 0x1D6A3, "Common", "Letter"},
+	{0x1D552, 0x1D6A5, "Common", "Letter"},
 	{0x1D6A8, 0x1D6C0, "Common", "Letter"},
 	{0x1D6C1, 0x1D6C1, "Common", "Symbol"},
 	{0x1D6C2, 0x1D6DA, "Common", "Letter"},
@@ -781,12 +887,10 @@
 	{0x1D7A9, 0x1D7A9, "Common", "Symbol"},
 	{0x1D7AA, 0x1D7C2, "Common", "Letter"},
 	{0x1D7C3, 0x1D7C3, "Common", "Symbol"},
-	{0x1D7C4, 0x1D7C9, "Common", "Letter"},
+	{0x1D7C4, 0x1D7CB, "Common", "Letter"},
 	{0x1D7CE, 0x1D7FF, "Common", "Number"},
 	{0x20000, 0x2A6D6, "Han", "Letter"},
 	{0x2F800, 0x2FA1D, "Han", "Letter"},
 	{0xE0001, 0xE0001, "Common", "Format"},
 	{0xE0020, 0xE007F, "Common", "Format"},
 	{0xE0100, 0xE01EF, "Inherited", "Mark"},
-	{0xF0000, 0xFFFFD, "Common", "Private Use"},
-	{0x100000, 0x10FFFD, "Common", "Private Use"},
