@@ -7,7 +7,10 @@
 #include "io.h"
 
 
-/* Expression definitions */
+/*======================================================================*\
+|*		Local constants and types				*|
+\*======================================================================*/
+
 #define NO_MATCH	0
 #define MATCH		1
 #define REG_ERROR	2
@@ -39,7 +42,10 @@ typedef struct regex REGEX;
 #define NIL_INT		((int *) 0)
 
 
-/* forward */
+/*======================================================================*\
+|*		Local function declarations and variables		*|
+\*======================================================================*/
+
 static char * substitute _((LINE * line, LINE * * line_poi, REGEX * program, 
 	int * old_lines_deleted, char * replacement, int * new_lines_inserted));
 static int compile _((character * pat_start, REGEX * program));
@@ -56,7 +62,6 @@ static LINE * search_it _((REGEX * program, FLAG method));
 static REGEX * make_expression _((char * expr));
 
 
-/* local */
 static FLAG wrapped_search;		/* temporary flag of wrapped search */
 
 static REGEX * lastprogram = NIL_REG;
@@ -108,9 +113,21 @@ SIDRV ()
   SIDF (REVERSE);
 }
 
-static
 void
-search_cur_char ()
+SCURCHARFW ()
+{
+  SCURCHAR (FORWARD);
+}
+
+void
+SCURCHARRV ()
+{
+  SCURCHAR (REVERSE);
+}
+
+void
+SCURCHAR (method)
+  FLAG method;
 {
   char cbuf [7];
   char * cp = cur_text;
@@ -125,7 +142,7 @@ search_cur_char ()
   }
   strncat (cbuf, cur_text, cp - cur_text);
 
-  search_for (cbuf, FORWARD);
+  search_for (cbuf, method);
 }
 
 /*
@@ -134,35 +151,10 @@ search_cur_char ()
 void
 SFW ()
 {
-  if ((keyshift & ctrlshift_mask) == ctrlshift_mask) {
-	keyshift = '0';
-	if (hop_flag > 0) {
-		Stag ();
-	} else {
-		LR ();
-	}
-  } else if (keyshift & ctrl_mask) {
-	keyshift = '0';
-	if (hop_flag > 0) {
-		hop_flag = 0;
-		search_cur_char ();
-	} else {
-		REPL ();
-	}
-  } else if (keyshift & shift_mask) {
-	keyshift = '0';
-	if (hop_flag > 0) {
-		hop_flag = 0;
-		Stag ();
-	} else {
-		GR ();
-	}
+  if (hop_flag > 0) {
+	SIDFW ();
   } else {
-	if (hop_flag > 0) {
-		SIDFW ();
-	} else {
-		search ("Search forward:", FORWARD);
-	}
+	search ("Search forward:", FORWARD);
   }
 }
 
@@ -203,27 +195,25 @@ reverse_method (dir)
 void
 RS ()
 {
-  if ((keyshift & altshift_mask) == altshift_mask) {
-	keyshift = '0';
-	SIDF (REVERSE);
-  } else if (keyshift & shift_mask) {
-	keyshift = '0';
-	SIDFW ();
-  } else if (keyshift & alt_mask) {
-	keyshift = '0';
-	if (hop_flag > 0) {
-		hop_flag = 0;
-		search_for (prevexpr, reverse_method (prevmethod));
-	} else {
-		do_search (lastprogram, reverse_method (lastmethod));
-	}
+  if (hop_flag > 0) {
+	hop_flag = 0;
+	prev_search ();
   } else {
-	if (hop_flag > 0) {
-		hop_flag = 0;
-		prev_search ();
-	} else {
-		re_search ();
-	}
+	re_search ();
+  }
+}
+
+/*
+ * RSreverse searches using the last search expression and reverse direction.
+ */
+void
+RSreverse ()
+{
+  if (hop_flag > 0) {
+	hop_flag = 0;
+	search_for (prevexpr, reverse_method (prevmethod));
+  } else {
+	do_search (lastprogram, reverse_method (lastmethod));
   }
 }
 
@@ -766,6 +756,9 @@ search_expr (expr, method)
  * search_corresponding searches for a corresponding bracket of a 
  * bracket pair expression.
  * The parameter 'method' must be valid.
+ * Special handling is triggered by special values of 'corresponding':
+   "/": XML tag matching
+   "#": #if/#else/#endif matching
  */
 void
 search_corresponding (expr, method, corresponding)
@@ -776,11 +769,14 @@ search_corresponding (expr, method, corresponding)
   register REGEX * program;
   LINE * match_line;
   int matched;
+  int nestingweight;
   int bracount = 0;
   LINE * old_line = cur_line;
   char * old_text = cur_text;
   FLAG was_redraw_pending;
-/*  char pflags [99]; */
+#ifdef debug_corresponding_redraw
+  char pflags [9999];
+#endif
 
 /* make the expression */
   if ((program = make_expression (expr)) == NIL_REG) {
@@ -788,7 +784,9 @@ search_corresponding (expr, method, corresponding)
   }
 
   redraw_pending = False;
-/* strcpy (pflags, redraw_pending ? "T" : "F"); */
+#ifdef debug_corresponding_redraw
+  strcpy (pflags, redraw_pending ? "T" : "F");
+#endif
   wrapped_search = False;
   do {
 	do {
@@ -803,11 +801,30 @@ search_corresponding (expr, method, corresponding)
 		}
 		/* now check if this is really a match */
 		if (* corresponding == '/') {
-			/* tag matching */
+			/* XML tag matching */
 			matched = white_space (* program->end_ptr)
 				|| * program->end_ptr == '\n'
 				|| * program->end_ptr == '>'
 				|| * program->end_ptr == '\0';
+		} else if (* corresponding == '#') {
+			/* #if/#else/#endif matching */
+			char * poi = program->start_ptr;
+			poi ++;
+			while (white_space (* poi) && * poi != '\n' && * poi != '\0') {
+				poi ++;
+			}
+			if (strisprefix ("if", poi)) {
+				matched = 1;
+				nestingweight = 1;
+			} else if (strisprefix ("els", poi)) {
+				matched = 1;
+				nestingweight = 0;
+			} else if (strisprefix ("end", poi)) {
+				matched = 1;
+				nestingweight = -1;
+			} else {
+				matched = 0;
+			}
 		} else {
 			matched = 1;
 		}
@@ -819,12 +836,21 @@ search_corresponding (expr, method, corresponding)
 	} while (quit == False && ! matched);
 
 	if (* corresponding == '/') {
-		/* tag matching */
+		/* XML tag matching */
 		if ((method == FORWARD) == (* (program->start_ptr + 1) == '/')) {
 			bracount --;
 		} else {
 			bracount ++;
 		}
+	} else if (* corresponding == '#') {
+		/* #if/#else/#endif matching */
+		if (bracount == 0 && nestingweight == 0) {
+			bracount = -1;
+		} else if (method == FORWARD) {
+			bracount += nestingweight;
+		} else {
+			bracount -= nestingweight;
+		}
 	} else {
 		if (strncmp (program->start_ptr, corresponding, strlen (corresponding)) == 0) {
 			bracount --;
@@ -832,17 +858,15 @@ search_corresponding (expr, method, corresponding)
 			bracount ++;
 		}
 	}
+
 	if (bracount >= 0) {
 	/* set position to match for next search: */
 		move_address_w_o_RD (program->start_ptr, find_y_w_o_RD (match_line));
-/* strcat (pflags, redraw_pending ? "T" : "F"); */
+#ifdef debug_corresponding_redraw
+		strcat (pflags, redraw_pending ? "T" : "F");
+#endif
 	}
-/*
-	build_string (text_buffer, "bracount = %d", bracount);
-	status_msg (text_buffer);
-	sleep (1);
-	break;
-*/
+
 	if (quit) {
 		status_msg ("Bracket search aborted");
 		quit = False;
@@ -853,15 +877,21 @@ search_corresponding (expr, method, corresponding)
 
 /* set position to old to enable correct function of find_y: */
   move_address_w_o_RD (old_text, find_y_w_o_RD (old_line));
-/* strcat (pflags, redraw_pending ? "T" : "F"); */
+#ifdef debug_corresponding_redraw
+  strcat (pflags, redraw_pending ? "T" : "F");
+#endif
 
 /* position and re-display if necessary: */
   was_redraw_pending = redraw_pending;
   move_address (program->start_ptr, find_y (match_line));
-/* strcat (pflags, redraw_pending ? "T" : "F"); */
+#ifdef debug_corresponding_redraw
+  strcat (pflags, redraw_pending ? "T" : "F");
+#endif
   if (was_redraw_pending && redraw_pending) {
 	RD ();
-/*	status_line ("pending redraw performed ", pflags);	*/
+#ifdef debug_corresponding_redraw
+	status_line ("pending redraw performed ", pflags);
+#endif
   }
 }
 
@@ -1206,7 +1236,7 @@ compile (pat_start, program)
 			/* Build list */
 			while (low_char <= high_char && low_char != 0) {
 				if (utf8_text) {
-					utflen = uniUTF (low_char, utfbuf);
+					utflen = utfencode (low_char, utfbuf);
 				} else if (cjk_text) {
 					utflen = cjkencode (low_char, utfbuf);
 				} else {
