@@ -1,5 +1,8 @@
+static struct interval list_Soft_Dotted [] =
+{
 	{0x0069, 0x006A},
 	{0x012F, 0x012F},
+	{0x0249, 0x0249},
 	{0x0268, 0x0268},
 	{0x029D, 0x029D},
 	{0x02B2, 0x02B2},
@@ -7,6 +10,9 @@
 	{0x0456, 0x0456},
 	{0x0458, 0x0458},
 	{0x1D62, 0x1D62},
+	{0x1D96, 0x1D96},
+	{0x1DA4, 0x1DA4},
+	{0x1DA8, 0x1DA8},
 	{0x1E2D, 0x1E2D},
 	{0x1ECB, 0x1ECB},
 	{0x2071, 0x2071},
@@ -24,3 +30,4 @@
 	{0x1D62A, 0x1D62B},
 	{0x1D65E, 0x1D65F},
 	{0x1D692, 0x1D693},
+};
